package cn.iocoder.yudao.module.sale.dal.dataobject.offer;

import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import com.baomidou.mybatisplus.annotation.*;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;

import javax.validation.constraints.NotNull;


/**
 * 报价单管理 DO
 *
 * @author 系统管理员
 */
@TableName("ecw_offer")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OfferDO extends BaseDO {

    /**
     * 主键ID
     */
    @TableId
    private Long offerId;
    /**
     * 报价单号
     */
    private String number;
    /**
     * 订单id
     */
    private Long orderId;
    /**
     * 订单编号
     */
    private String orderNo;
    /**
     * 客户id，发货人
     */
    private Long consignorId;
    /**
     * 收货人id
     */
    private Long consigneeId;
    /**
     * 唛头
     */
    private String marks;

    /**
     * 订单类型(可多选)：1 集运服务 2 海外仓
     */
    private String serviceType;

    /**
     * 报关类别：我司全代：1，自单代报：2，混合报关：3
     */
    private Integer customsType;
    /**
     * 始发地
     */
    private Long departureId;
    /**
     * 始发地信息
     */
    private String departure;

    @ApiModelProperty(value = "目的地国ID")
    private Long destCountryId;
    /**
     * 目的地
     */
    private Long objectiveId;
    /**
     * 目的地信息
     */
    private String objective;
    /**
     * 预计结束时间
     */
    private Date stopTime;
    /**
     * 控货
     */
    private Boolean control;
    /**
     * 竞争对手
     */
    private String competitor;
    /**
     * 重要程度
     */
    private Integer importance;
    /**
     * 价格有效期开始
     */
    private Date startTime;
    /**
     * 价格有效期结束
     */
    private Date endTime;
    /**
     * 交货方式：FOB(离岸价)，CIF(到岸价)，CNF(成本加运费)，EXW(出厂价)
     */
    private Integer tradeType;
    /**
     * 交货港口、地点
     */
    private String tradeAdress;
    /**
     * 备注
     */
    private String remarks;
    /**
     * 状态：取消报价：0，特价审批中：1，草稿：2, 需求确认：3，赢单：4，输单：5，报价完成：6 跟进中：7
     */
    private Integer status;
    /**
     * 取消报价前状态
     */
    private Integer oldStatus;
    /**
     * 报价关联客户，发货人：1，收货人：2
     */
    private Integer relation;
    /**
     * 报价关联客户id
     */
    private Long relationId;

    @ApiModelProperty(value = "客户经理ID")
    private Long salesmanId;

    @ApiModelProperty(value = "客户经理所属部门ID")
    private Long deptId;

    /**
     * 佣金类型：0 无 1 明佣 2 暗佣 3 明+暗
     */
    private Integer commissionType;
    /**
     * 入仓类型类型
     */
    private Integer warehousingType;
    /**
     * 原因
     */
    private String reason;
    /**
     * 预计费用清单
     */
    private String estCost;

    /**
     * 站内信状态，0未发送，1已发送
     */
    private Boolean sendstatus;

    /**
     * 运输id
     */
    private Integer transportId;


    @ApiModelProperty(value = "线路id")
    private Long lineId;

    @ApiModelProperty(value = "渠道id")
    private Long channelId;

    @ApiModelProperty(value = "跟进业务员")
    private String followUpSalesmanId;


    @ApiModelProperty(value = "其他费用")
    @TableField(exist = false)
    private BigDecimal otherFee;

    @ApiModelProperty(value = "其他费用货币ID")
    @TableField(exist = false)
    private Integer otherFeeCurrencyId;
}
