package cn.iocoder.yudao.module.sale.dal.dataobject.offer;

import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import com.baomidou.mybatisplus.annotation.*;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;

/**
 * 报价商品详情 DO
 *
 * @author 系统管理员
 */
@TableName("ecw_offer_prod")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class OfferProdDO extends BaseDO {

    /**
     * 主键ID
     */
    @TableId
    private Long offerProdId;

    /**
     * 运输方式ID
     */
    private Integer transportId;
    /**
     * 报价单ID
     */
    private Long offerId;
    /**
     * 商品ID
     */
    private Long prodId;
    /**
     * 产品中文标题
     */
    private String prodTitleZh;
    /**
     * 产品英文标题
     */
    private String prodTitleEn;

    @ApiModelProperty(value = "商品第三方链接")
    private String link;
    /**
     * 线路id
     */
    private Long lineId;
    /**
     * 渠道id
     */
    private Long channelId;
    /**
     * 产品类型
     */
    private Integer productType;
    /**
     * 品牌(0 否 1 是)
     */
    private Integer brand;
    /**
     * 件数
     */
    private Integer num;
    /**
     * 单位
     */
    private String unit;
    /**
     * 箱规
     */
    private String boxGauge;
    /**
     * 体积
     */
    private BigDecimal volume;
    /**
     * 重量
     */
    private BigDecimal weight;
    /**
     * 货值
     */
    private BigDecimal worth;
    /**
     * 货值金额单位ID
     */
    private Integer worthCurrency;
    /**
     * 商品特性(可选多个特性ids)
     */
    private String prodAttrIds;

    @ApiModelProperty(value = "是否特价")
    private Boolean specialPriceType;
    /**
     * 运费单价
     */
    private BigDecimal oneSeaFreight;
    /**
     * 运费
     */
    private BigDecimal seaFreight;
    /**
     * 原来运费
     */
    private BigDecimal originalSeaFreight;
    /**
     * 运费用金额单位ID
     */
    private Integer seaFreightCurrency;
    /**
     * 运费体积单位
     */
    private Long seaFreightVolume;
    /**
     * 清关费单价
     */
    private BigDecimal oneClearanceFreight;
    /**
     * 清关费
     */
    private BigDecimal clearanceFreight;
    /**
     * 原来清关费
     */
    private BigDecimal originalClearanceFreight;
    /**
     * 清关费用金额单位ID
     */
    private Integer clearanceFreightCurrency;
    /**
     * 清关费体积单位
     */
    private Long clearanceFreightVolume;
    /**
     * 预计销售额
     */
    private BigDecimal estimatedSales;

    @ApiModelProperty(value = "最小计量重量")
    private BigDecimal minMeteringWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "最小计量体积")
    private BigDecimal minMeteringVolume  = BigDecimal.ZERO;

    @ApiModelProperty(value = "最小计量个数")
    private BigDecimal minMeteringQuantity  = BigDecimal.ZERO;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ApiModelProperty(value = "计费方式：0 运费/清关费 1 全包价")
    private Integer charging;

    @ApiModelProperty(value = "数量")
    private Integer quantity;
    /**
     * 状态：-1 未设置价格 0 正常 1 审批中
     */
    private Integer status;
    /**
     * 价格有效期开始
     */
    private Date startTime;
    /**
     * 价格有效期结束
     */
    private Date endTime;

    @ApiModelProperty("快递单号")
    private String expressNo;

    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ApiModelProperty("明佣金额")
    private BigDecimal plainCommission = BigDecimal.ZERO;

    @ApiModelProperty("暗佣金额")
    private BigDecimal hiddenCommission = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠运费金额(原始单价上进行优惠)")
    private BigDecimal discountFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠运费总金额")
    private BigDecimal discountFreightTotalPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠清关费金额(原始单价上进行优惠)")
    private BigDecimal discountClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠清关费总金额")
    private BigDecimal discountClearanceFeeTotalPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "液体清关费单价")
    private BigDecimal liquidClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "液体运费单价")
    private BigDecimal liquidFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "带电清关费单价")
    private BigDecimal electrifiedClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "带电运费单价")
    private BigDecimal electrifiedFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "品牌清关费单价")
    private BigDecimal brandClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "品牌运费单价")
    private BigDecimal brandFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "包装清关费单价")
    private BigDecimal packagingClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "包装运费单价")
    private BigDecimal packagingFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "是否特价佣金设置")
    private Boolean specialCommission = Boolean.FALSE;

    @ApiModelProperty(value = "使用的优惠信息")
    private String couponInfo;

    @ApiModelProperty(value = "渠道清关费用")
    @TableField(exist = false)
    private BigDecimal channelClearancePrice;

    @ApiModelProperty(value = "渠道运费费用")
    @TableField(exist = false)
    private BigDecimal channelSeaPrice;
}
