package cn.iocoder.yudao.module.sale.dal.mysql.channel;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.module.sale.dal.dataobject.channel.ChannelDO;
import org.apache.ibatis.annotations.*;
import cn.iocoder.yudao.module.sale.vo.channel.*;

/**
 * 渠道管理 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface ChannelMapper extends BaseMapperX<ChannelDO> {

    default PageResult<ChannelDO> selectPage(ChannelPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<ChannelDO>()
                .eqIfPresent(ChannelDO::getNameZh, reqVO.getNameZh())
                .eqIfPresent(ChannelDO::getNameEn, reqVO.getNameEn())
                .eqIfPresent(ChannelDO::getInternalNameZh, reqVO.getInternalNameZh())
                .eqIfPresent(ChannelDO::getInternalNameEn, reqVO.getInternalNameEn())
                .eqIfPresent(ChannelDO::getTypeNumber, reqVO.getTypeNumber())
                .eqIfPresent(ChannelDO::getCode, reqVO.getCode())
                .eqIfPresent(ChannelDO::getCountryId, reqVO.getCountryId())
                .eqIfPresent(ChannelDO::getWarehouseIds, reqVO.getWarehouseIds())
                .eqIfPresent(ChannelDO::getSort, reqVO.getSort())
                .eqIfPresent(ChannelDO::getExpressId, reqVO.getExpressId())
                .eqIfPresent(ChannelDO::getEtaTime, reqVO.getEtaTime())
                .eqIfPresent(ChannelDO::getChannelAgent, reqVO.getChannelAgent())
                .eqIfPresent(ChannelDO::getRemarksZh, reqVO.getRemarksZh())
                .eqIfPresent(ChannelDO::getRemarksEn, reqVO.getRemarksEn())
                .eqIfPresent(ChannelDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(ChannelDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(ChannelDO::getChannelId));
    }

    default List<ChannelDO> selectList(ChannelExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<ChannelDO>()
                .eqIfPresent(ChannelDO::getNameZh, reqVO.getNameZh())
                .eqIfPresent(ChannelDO::getNameEn, reqVO.getNameEn())
                .eqIfPresent(ChannelDO::getInternalNameZh, reqVO.getInternalNameZh())
                .eqIfPresent(ChannelDO::getInternalNameEn, reqVO.getInternalNameEn())
                .eqIfPresent(ChannelDO::getTypeNumber, reqVO.getTypeNumber())
                .eqIfPresent(ChannelDO::getCode, reqVO.getCode())
                .eqIfPresent(ChannelDO::getCountryId, reqVO.getCountryId())
                .eqIfPresent(ChannelDO::getWarehouseIds, reqVO.getWarehouseIds())
                .eqIfPresent(ChannelDO::getSort, reqVO.getSort())
                .eqIfPresent(ChannelDO::getExpressId, reqVO.getExpressId())
                .eqIfPresent(ChannelDO::getEtaTime, reqVO.getEtaTime())
                .eqIfPresent(ChannelDO::getChannelAgent, reqVO.getChannelAgent())
                .eqIfPresent(ChannelDO::getRemarksZh, reqVO.getRemarksZh())
                .eqIfPresent(ChannelDO::getRemarksEn, reqVO.getRemarksEn())
                .eqIfPresent(ChannelDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(ChannelDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(ChannelDO::getChannelId));
    }

    @ResultType(List.class)
    @Select({
            "<script>",
            "select c.*,",
            "e.company_name, ",
            "r.title_zh as country_title_zh, r.title_en as country_title_en, ",
            "cr.nickname as creator_name, ",
            "up.nickname as updater_name ",
            "from ecw_channel c ",
            "left join ecw_express e on e.id = c.express_id ",
            "left join ecw_region r on r.id = c.country_id ",
            "left join system_user cr on cr.id = c.creator ",
            "left join system_user up on up.id = c.updater ",
            "where c.deleted = 0",
            "<when test = 'pageVO.status != null'>",
            "AND c.status = #{pageVO.status}",
            "</when>",
            "<when test = 'pageVO.countryId != null'>",
            "AND c.country_id = #{pageVO.countryId}",
            "</when>",
            "<when test = 'pageVO.etaTime != null'>",
            "AND c.eta_time = #{pageVO.etaTime}",
            "</when>",
            "<when test = 'pageVO.nameZh != null and lang != 1'>",
            "AND c.name_zh like concat('%',concat(#{pageVO.nameZh},'%'))",
            "</when>",
            "<when test = 'pageVO.nameEn != null and lang != 0'>",
            "AND c.name_en like concat('%',concat(#{pageVO.nameEn},'%'))",
            "</when>",
            "<when test = 'pageVO.internalNameZh != null and lang != 1'>",
            "AND c.internal_name_zh like concat('%',concat(#{pageVO.internalNameZh},'%'))",
            "</when>",
            "<when test = 'pageVO.internalNameEn != null and lang != 0'>",
            "AND c.internal_name_en like concat('%',concat(#{pageVO.internalNameEn},'%'))",
            "</when>",
            "<when test = 'pageVO.remarksZh != null and lang != 1'>",
            "AND c.remarks_zh like concat('%',concat(#{pageVO.remarksZh},'%'))",
            "</when>",
            "<when test = 'pageVO.remarksEn != null and lang != 0'>",
            "AND c.remarks_en like concat('%',concat(#{pageVO.remarksEn},'%'))",
            "</when>",
            "<when test = 'pageVO.code != null and pageVO.code != \"\"'>",
            "AND c.code like concat('%',concat(#{pageVO.code},'%'))",
            "</when>",
            "<when test = 'pageVO.channelAgent != null'>",
            "AND c.channel_agent like concat('%',concat(#{pageVO.channelAgent},'%'))",
            "</when>",
            "<when test = 'pageVO.typeNumber != null'>",
            "AND c.type_number like concat('%',concat(#{pageVO.typeNumber},'%'))",
            "</when>",
            "<when test = 'pageVO.expressId != null'>",
            "AND c.express_id = #{pageVO.expressId} ",
            "</when>",
            " order by c.sort ",
            "limit #{pageVO.pageNo}, #{pageVO.pageSize}",
            "</script>"
    })
    List<ChannelRespVO> channelPage(@Param("pageVO") ChannelPageReqVO reqVO, @Param("lang") Integer lang);

    @ResultType(Long.class)
    @Select({
            "<script>",
            "select count(1)",
            "from ecw_channel c ",
            "left join ecw_express e on e.id = c.express_id ",
            "where c.deleted = 0",
            "<when test = 'pageVO.status != null'>",
            "AND c.status = #{pageVO.status}",
            "</when>",
            "<when test = 'pageVO.countryId != null'>",
            "AND c.country_id = #{pageVO.countryId}",
            "</when>",
            "<when test = 'pageVO.etaTime != null'>",
            "AND c.eta_time = #{pageVO.etaTime}",
            "</when>",
            "<when test = 'pageVO.nameZh != null and pageVO.lang != 1'>",
            "AND c.name_zh like concat('%',concat(#{pageVO.nameZh},'%'))",
            "</when>",
            "<when test = 'pageVO.nameEn != null and pageVO.lang != 0'>",
            "AND c.name_en like concat('%',concat(#{pageVO.nameEn},'%'))",
            "</when>",
            "<when test = 'pageVO.internalNameZh != null and pageVO.lang != 1'>",
            "AND c.internal_name_zh like concat('%',concat(#{pageVO.internalNameZh},'%'))",
            "</when>",
            "<when test = 'pageVO.internalNameEn != null and pageVO.lang != 0'>",
            "AND c.internal_name_en like concat('%',concat(#{pageVO.internalNameEn},'%'))",
            "</when>",
            "<when test = 'pageVO.remarksZh != null and pageVO.lang != 1'>",
            "AND c.remarks_zh like concat('%',concat(#{pageVO.remarksZh},'%'))",
            "</when>",
            "<when test = 'pageVO.remarksEn != null and pageVO.lang != 0'>",
            "AND c.remarks_en like concat('%',concat(#{pageVO.remarksEn},'%'))",
            "</when>",
            "<when test = 'pageVO.channelAgent != null'>",
            "AND c.channel_agent like concat('%',concat(#{pageVO.channelAgent},'%'))",
            "</when>",
            "<when test = 'pageVO.typeNumber != null'>",
            "AND c.type_number like concat('%',concat(#{pageVO.typeNumber},'%'))",
            "</when>",
            "<when test = 'pageVO.expressId != null'>",
            "AND c.express_id = #{pageVO.expressId} ",
            "</when>",
            "</script>"
    })
    Long countChannelList(@Param("pageVO") ChannelPageReqVO pageVO);

    @Update("update ecw_channel set `status` = !`status` where `channel_id` = #{channelId}")
    void updateChannelStatus(@Param("channelId") Long channelId);

}
