package cn.iocoder.yudao.module.sale.dal.mysql.deptTarget;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.DeptTargetDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.PersonalDeptTargetDO;
import cn.iocoder.yudao.module.sale.dto.AchievementStatisticsDto;
import cn.iocoder.yudao.module.sale.dto.MyAchievementDto;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import cn.iocoder.yudao.module.sale.vo.deptTarget.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 部门业绩目标设置 Mapper
 * @author 系统管理员
 */
@Mapper
public interface DeptTargetMapper extends AbstractMapper<DeptTargetDO> {
    @Override
    default PageResult<DeptTargetDO> selectPage(PageVO page, Object object) {
        if (object instanceof DeptTargetQueryVO) {
            DeptTargetQueryVO vo = (DeptTargetQueryVO)object;
            return selectPage(page, new LambdaQuery<DeptTargetDO>()
                .betweenIfPresent(DeptTargetDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                .eqIfPresent(DeptTargetDO::getDeptId, vo.getDeptId())
                .eqIfPresent(DeptTargetDO::getTargetType, vo.getTargetType())
                .likeIfPresent(DeptTargetDO::getTargetPeriodName, vo.getTargetPeriodName())
                .betweenIfPresent(DeptTargetDO::getStartTime, vo.getBeginStartTime(), vo.getEndStartTime())
                .betweenIfPresent(DeptTargetDO::getEndTime, vo.getBeginEndTime(), vo.getEndEndTime())
                .eqIfPresent(DeptTargetDO::getTransportType, vo.getTransportType())
                .eqIfPresent(DeptTargetDO::getShippingChannel, vo.getShippingChannel())
                .eqIfPresent(DeptTargetDO::getCubeNum, vo.getCubeNum())
                    .orderByDesc(DeptTargetDO::getId));
        }

        return null;
    }

    @Override
    default List<DeptTargetDO> selectList(Object object) {
        if (object instanceof DeptTargetQueryVO) {
            DeptTargetQueryVO vo = (DeptTargetQueryVO)object;
            return selectList(new LambdaQuery<DeptTargetDO>()
                .betweenIfPresent(DeptTargetDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                .eqIfPresent(DeptTargetDO::getDeptId, vo.getDeptId())
                .eqIfPresent(DeptTargetDO::getTargetType, vo.getTargetType())
                .likeIfPresent(DeptTargetDO::getTargetPeriodName, vo.getTargetPeriodName())
                .betweenIfPresent(DeptTargetDO::getStartTime, vo.getBeginStartTime(), vo.getEndStartTime())
                .betweenIfPresent(DeptTargetDO::getEndTime, vo.getBeginEndTime(), vo.getEndEndTime())
                .eqIfPresent(DeptTargetDO::getTransportType, vo.getTransportType())
                .eqIfPresent(DeptTargetDO::getShippingChannel, vo.getShippingChannel())
                .eqIfPresent(DeptTargetDO::getCubeNum, vo.getCubeNum())
                    .orderByDesc(DeptTargetDO::getId));
        }

        return null;
    }

    
    @ResultType(DeptTargetBackVO.class)
    @Select({
            "<script>",
            "SELECT" ,
            " t1.id as id,",
            " t2.`name` as deptName," ,
            " t1.target_type as targetType,"  ,
            " DATE_FORMAT(t1.start_time,'%Y-%m-%d') as startTime,"  ,
            " t1.target_period_name as targetPeriodName,"  ,
            " DATE_FORMAT(t1.end_time,'%Y-%m-%d') as endTime,"  ,
            " (SELECT  count( 1 )  FROM  system_user t  WHERE  t.dept_id = t1.dept_id  ) as deptEmpCount,"  ,
            " t1.cube_num as cubeNum,"  ,
            " t1.cube_unit as cubeUnit,"  ,
            " t1.transport_type as transportType,"  ,
            " t1.shipping_channel as shippingChannel,"  ,
            " IFNULL(( select sum(t.complete_volume) from ecw_target_log t LEFT JOIN system_user a on t.user_id=a.id where a.dept_id=t1.dept_id and t.deleted = 0  and t.achievement_time &gt;= t1.start_time and t.achievement_time &lt;= t1.end_time ),0 ) as completeCubeNum"  ,
            " FROM ecw_dept_target t1"  ,
            " LEFT JOIN system_dept t2 ON t1.dept_id = t2.id",
            " where t1.deleted = 0",
            "<when test = 'query.deptId != null'>",
            "AND t1.dept_id = #{query.deptId}",
            "</when>",
            "<when test = 'query.flag != null and query.flag == \"my\"'>",
            "AND t1.dept_id in",
            "<foreach item='param' index='index' collection='query.deptIds' open='(' separator=',' close=')'>",
            "#{param}",
            "</foreach>",
            "</when>",
            "<when test = 'query.targetType != null'>",
            "AND t1.target_type = #{query.targetType}",
            "</when>",
            "<when test = 'query.transportType != null'>",
            "AND t1.transport_type = #{query.transportType}",
            "</when>",
            "<when test = 'query.shippingChannel != null'>",
            "AND t1.shipping_channel = #{query.shippingChannel}",
            "</when>",
            "</script>"
    })
    IPage<DeptTargetBackVO> getDeptTargetStatisticsPage(@Param("mpPage") IPage<DeptTargetBackVO> mpPage, @Param("query") DeptTargetQueryVO query);

    IPage<PersonalDeptTargetDO> getPersonalDeptTargetPage(@Param("query")DeptTargetQueryVO query, @Param("mpPage") IPage<PersonalDeptTargetDO> mpPage);

    IPage<MyAchievementDto> getMyAchievementByPage(@Param("query")DeptTargetQueryVO query, @Param("mpPage")IPage<MyAchievementDto> mpPage);

    List<MyAchievementDto> getMyAchievement(@Param("query")DeptTargetQueryVO query);

    IPage<MyAchievementDto> getMyDeptAchievementByPage(@Param("query")DeptTargetQueryVO query, @Param("mpPage")IPage<MyAchievementDto> mpPage);

    List<MyAchievementDto> getMyDeptAchievement(@Param("query")DeptTargetQueryVO query);

    List<AchievementStatisticsDto> getExportStatisticsExcel(@Param("query")DeptTargetQueryVO exportReqVO);
}
