package cn.iocoder.yudao.module.sale.dal.mysql.offer;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferLogDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.sale.vo.offer.OfferPageReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.OfferRespVO;
import cn.iocoder.yudao.module.sale.vo.offer.log.OfferLogBackVO;
import cn.iocoder.yudao.module.sale.vo.offer.log.OfferLogQueryVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 报价单跟进日志 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface OfferLogMapper extends AbstractMapper<OfferLogDO> {
    @Override
    default PageResult<OfferLogDO> selectPage(PageVO page, Object object) {
        if (object instanceof OfferLogQueryVO) {
            OfferLogQueryVO vo = (OfferLogQueryVO) object;
            return selectPage(page, new LambdaQuery<OfferLogDO>()
                    .eqIfPresent(OfferLogDO::getOfferId, vo.getOfferId())
                    .eqIfPresent(OfferLogDO::getNumber, vo.getNumber())
                    .eqIfPresent(OfferLogDO::getType, vo.getType())
                    .betweenIfPresent(OfferLogDO::getFollowUpTime, vo.getBeginFollowUpTime(), vo.getEndFollowUpTime())
                    .eqIfPresent(OfferLogDO::getFollowUpSalesmanId, vo.getFollowUpSalesmanId())
                    .eqIfPresent(OfferLogDO::getContacts, vo.getContacts())
                    .eqIfPresent(OfferLogDO::getFollowUpMethod, vo.getFollowUpMethod())
                    .eqIfPresent(OfferLogDO::getCustomerFeedback, vo.getCustomerFeedback())
                    .eqIfPresent(OfferLogDO::getProcessingResults, vo.getProcessingResults())
                    .betweenIfPresent(OfferLogDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(OfferLogDO::getOfferLogId));
        }

        return null;
    }

    @Override
    default List<OfferLogDO> selectList(Object object) {
        if (object instanceof OfferLogQueryVO) {
            OfferLogQueryVO vo = (OfferLogQueryVO) object;
            return selectList(new LambdaQuery<OfferLogDO>()
                    .eqIfPresent(OfferLogDO::getOfferId, vo.getOfferId())
                    .eqIfPresent(OfferLogDO::getNumber, vo.getNumber())
                    .eqIfPresent(OfferLogDO::getType, vo.getType())
                    .betweenIfPresent(OfferLogDO::getFollowUpTime, vo.getBeginFollowUpTime(), vo.getEndFollowUpTime())
                    .eqIfPresent(OfferLogDO::getFollowUpSalesmanId, vo.getFollowUpSalesmanId())
                    .eqIfPresent(OfferLogDO::getContacts, vo.getContacts())
                    .eqIfPresent(OfferLogDO::getFollowUpMethod, vo.getFollowUpMethod())
                    .eqIfPresent(OfferLogDO::getCustomerFeedback, vo.getCustomerFeedback())
                    .eqIfPresent(OfferLogDO::getProcessingResults, vo.getProcessingResults())
                    .betweenIfPresent(OfferLogDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(OfferLogDO::getOfferLogId));
        }

        return null;
    }


    @ResultType(OfferRespVO.class)
    @Select({
            "<script>",
            "select o.offer_id, ",
            "o.number, ",
            "o.order_no, ",
            "if(#{lang} = 0, o.departure ->> '$.titleZh', o.departure ->> '$.titleEn') as departure_name, ",
            "if(#{lang} = 0, o.objective ->> '$.titleZh', o.objective ->> '$.titleEn') as objective_name, ",
            "o.start_time, ",
            "o.end_time, ",
            "o.stop_time, ",
            "o.status, ",
            "o.transport_ids, ",
            "o.est_cost, ",
            "o.create_time, ",
            "o.update_time, ",
            "u.username as creator_name, ",
            "c.name as relation_name",
            "from ecw_offer o ",
            "left join ecw_customer c on c.id = o.relation_id ",
            "left join system_user u on u.id = o.creator ",
            "where o.deleted = 0",
            "<when test = 'pageVO.status != null'>",
            "AND `o.status` = #{pageVO.status}",
            "</when>",
            "<when test = 'pageVO.number != null and pageVO.number != \"\" '>",
            "AND `o.number` like concat('%',concat(#{pageVO.number},'%'))",
            "</when>",
            "<when test = 'pageVO.orderNo != null and pageVO.orderNo != \"\" '>",
            "AND `o.order_no` like concat('%',concat(#{pageVO.orderNo},'%'))",
            "</when>",
            "<when test = 'pageVO.relationId != null'>",
            "AND `o.relation_id` = #{pageVO.relationId}",
            "</when>",
            " order by o.offer_id DESC ",
            "limit #{pageVO.pageNo}, #{pageVO.pageSize}",
            "</script>"
    })
    List<OfferRespVO> offerLogPage(@Param("pageVO") OfferPageReqVO pageVO, @Param("lang") Integer lang);

    @ResultType(OfferLogBackVO.class)
    @Select({
            "<script>",
            "select ",
            "o.*, ",
            "eo.order_no, ",
            "if(#{query.lang} = 0, eo.departure ->> '$.titleZh', eo.departure ->> '$.titleEn') as departure_name, ",
            "if(#{query.lang} = 0, eo.objective ->> '$.titleZh', eo.objective ->> '$.titleEn') as objective_name, ",
            "eo.start_time, ",
            "eo.end_time, ",
            "eo.stop_time, ",
            "eo.status, ",
            "eo.transport_id, ",
            "eo.est_cost, ",
            "u.username as creator_name, ",
            "c.name as relation_name",
            "from ecw_offer_log o ",
            "left join ecw_offer eo on eo.offer_id = o.offer_id ",
            "left join ecw_customer c on c.id = eo.relation_id ",
            "left join system_user u on u.id = o.creator ",
            "where eo.deleted = 0",
            "<when test = 'query.type != null'>",
            "AND o.`type` = #{query.type}",
            "</when>",
            "<when test = 'query.number != null and query.number != \"\" '>",
            "AND o.`number` like concat('%',concat(#{query.number},'%'))",
            "</when>",
            "<when test = 'query.beginFollowUpTime != null and query.endFollowUpTime != null '>",
            "AND o.`follow_up_time` between #{beginFollowUpTime} and #{endFollowUpTime}",
            "</when>",
            "<when test = 'query.customerId != null'>",
            "AND eo.`relation_id` = #{query.customerId}",
            "</when>",
            "</script>"
    })
    IPage<OfferLogBackVO> getCustomerOfferLogPage(@Param("mpPage") IPage<OfferLogBackVO> mpPage, @Param("query") OfferLogQueryVO query);
//
//    @ResultType(Long.class)
//    @Select({
//            "<script>",
//            "select count(1)",
//            "from ecw_offer o ",
//            "left join ecw_customer c on c.id = o.relation_id ",
//            "where o.deleted = 0",
//            "<when test = 'pageVO.status != null'>",
//            "AND `o.status` = #{pageVO.status}",
//            "</when>",
//            "<when test = 'pageVO.number != null and pageVO.number != \"\" '>",
//            "AND `o.number` like concat('%',concat(#{pageVO.number},'%'))",
//            "</when>",
//            "<when test = 'pageVO.orderNo != null and pageVO.orderNo != \"\" '>",
//            "AND `o.order_no` like concat('%',concat(#{pageVO.orderNo},'%'))",
//            "</when>",
//            "<when test = 'pageVO.relationId != null'>",
//            "AND `o.relation_id` = #{pageVO.relationId}",
//            "</when>",
//            " order by o.offer_id DESC ",
//            "limit #{pageVO.pageNo}, #{pageVO.pageSize}",
//            "</script>"
//    })
//    Long countOfferLogList(@Param("pageVO") OfferPageReqVO pageVO);
}
