
package cn.iocoder.yudao.module.sale.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.CustomerFollowupExcelExportPushEvent;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.customer.convert.customerFollowup.CustomerFollowupConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerFollowup.CustomerFollowupDO;
import cn.iocoder.yudao.module.customer.service.customerFollowup.CustomerFollowupService;
import cn.iocoder.yudao.module.customer.vo.customerFollowup.CustomerFollowupBackVO;
import cn.iocoder.yudao.module.customer.vo.customerFollowup.CustomerFollowupExcelVO;
import cn.iocoder.yudao.module.customer.vo.customerFollowup.CustomerFollowupQueryVO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;


/**
 * 间接客户excel导出监听
 *
 * @author zhengYi
 */
@Component("CustomerFollowupExcelExportListener")
@AllArgsConstructor
@Slf4j
public class CustomerFollowupExcelExportListener {


    private UeProperties ueProperties;
    private FileService fileService;


    private final CustomerFollowupService customerFollowupService;


    /**
     * 客户纪录导出监听
     *
     * @param event 编号记录
     */
    @EventListener(CustomerFollowupExcelExportPushEvent.class)
    public void customerIndirectExcelExportPushEvent(CustomerFollowupExcelExportPushEvent event) {
        CustomerFollowupQueryVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), CustomerFollowupQueryVO.class);
        List<CustomerFollowupDO> list = customerFollowupService.getFollowupList(exportReqVO);

        makeExcelUpload(event, list);
    }



    public void makeExcelUpload(CustomerFollowupExcelExportPushEvent event, List<CustomerFollowupDO> customerFollowupDOList) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                if(CollectionUtil.isEmpty(customerFollowupDOList)) return;


                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/customer/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("followup.xlsx");


                // 导出 Excel
                List<CustomerFollowupExcelVO> datas = CustomerFollowupConvert.INSTANCE.convertList2(customerFollowupDOList);

                String path = ExcelUtils.write(dir, fileName, "数据", CustomerFollowupExcelVO.class, datas);

                // 获取到临时文件
                File file = new File(path);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());

            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }




}
