package cn.iocoder.yudao.module.sale.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.IndirectCustomerExcelExportPushEvent;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.customer.convert.indirectCustomer.IndirectCustomerConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomer.IndirectCustomerDO;
import cn.iocoder.yudao.module.customer.dto.indirectCustomer.IndirectCustomerExportReqDTO;
import cn.iocoder.yudao.module.customer.dto.indirectCustomer.IndirectCustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.service.indirectCustomer.IndirectCustomerService;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.IndirectCustomerExcelVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.IndirectCustomerExportReqVO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;


/**
 * 间接客户excel导出监听
 *
 * @author zhengYi
 */
@Component("IndirectCustomerExcelExportListener")
@AllArgsConstructor
@Slf4j
public class IndirectCustomerExcelExportListener {


    private UeProperties ueProperties;
    private FileService fileService;


    private final IndirectCustomerService indirectCustomerService;


    /**
     * 间接客户导出监听
     *
     * @param event 编号记录
     */
    @EventListener(IndirectCustomerExcelExportPushEvent.class)
    public void customerIndirectExcelExportPushEvent(IndirectCustomerExcelExportPushEvent event) {
        //IndirectCustomerExportReqVO exportReqVO = JSONObject.parseObject(event.getRequestParams(), IndirectCustomerExportReqVO.class);
        IndirectCustomerPageReqDTO exportReqVO = JSONObject.parseObject(event.getRequestParams(), IndirectCustomerPageReqDTO.class);
        List<IndirectCustomerDO> list = indirectCustomerService.getIndirectCustomerListReq(exportReqVO);

        makeExcelUpload(event, list);
    }



    public void makeExcelUpload(IndirectCustomerExcelExportPushEvent event, List<IndirectCustomerDO> indirectCustomerDOList) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                if(CollectionUtil.isEmpty(indirectCustomerDOList)) return;


                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/customer/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("indirectCustomer.xlsx");


                // 导出 Excel
                List<IndirectCustomerExcelVO> datas = IndirectCustomerConvert.INSTANCE.convertExcelList(indirectCustomerDOList);

                String path = ExcelUtils.write(dir, fileName, "数据", IndirectCustomerExcelVO.class, datas);

                // 获取到临时文件
                File file = new File(path);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());

            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }




}
