package cn.iocoder.yudao.module.sale.listener;


import cn.iocoder.yudao.framework.apollo.core.event.OfferAssociationOrderEvent;
import cn.iocoder.yudao.framework.apollo.core.event.OfferUpdateOrderNumberEvent;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.module.sale.service.offer.OfferService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.AllArgsConstructor;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.sale.enums.ErrorCodeConstants.*;

/**
 * 编号记录监听
 *
 * @author zhengYi
 */
@Component("OfferAssociationOrderListener")
@AllArgsConstructor
public class OfferAssociationOrderListener {

    private final OfferService offerService;

    /**
     * 有最新编号记录则执行修改或插入操作，无最新编号记录执行查询最新编号记录操作
     *
     * @param event 关联参数
     */
    @EventListener(OfferAssociationOrderEvent.class)
    public void OfferUpdateOrderNumber(OfferAssociationOrderEvent event) {
        OfferDO offerDO = offerService.getOffer(event.getOfferId());
        if (Objects.isNull(offerDO)) {
            throw exception(OFFER_NOT_EXISTS);
        }
        if (offerDO.getStatus() == 1) {
            throw exception(OFFER_STATUS_ERROR);
        }
        if (Objects.nonNull(offerDO.getOrderId()) && offerDO.getOrderId() > 0L) {
            if (!Objects.equals(offerDO.getOrderId(), event.getOrderId())) {
                throw exception(ORDER_ASSOCIATION_OFFER_NO_CAN_UPDATE);
            }
        }
        offerDO.setOrderNo(event.getOrderNo());
        offerDO.setOrderId(event.getOrderId());
        offerDO.setStatus(4);
        offerDO.setUpdater(event.getOperator());
        offerDO.setUpdateTime(new Date());
        offerService.updateById(offerDO);
        offerService.addOfferLog(offerDO.getOfferId(), offerDO.getNumber(), offerDO.getRelationId(), 6, event.getOperator(), "报价单赢单");
    }
}
