package cn.iocoder.yudao.module.sale.listener;


import cn.iocoder.yudao.framework.apollo.core.event.OfferUpdateOrderNumberEvent;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.module.sale.service.offer.OfferService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.AllArgsConstructor;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;

/**
 * 编号记录监听
 *
 * @author zhengYi
 */
@Component("OfferUpdateOrderNumberListener")
@AllArgsConstructor
public class OfferUpdateOrderNumberListener {

    private final OfferService offerService;

    /**
     * 有最新编号记录则执行修改或插入操作，无最新编号记录执行查询最新编号记录操作
     *
     * @param event 编号记录
     */
    @EventListener(OfferUpdateOrderNumberEvent.class)
    public void OfferUpdateOrderNumber(OfferUpdateOrderNumberEvent event) {
        offerService.update(new LambdaUpdateWrapper<OfferDO>()
                .set(OfferDO::getOrderNo, event.getOrderNo())
                .eq(OfferDO::getStatus, 4)
                .eq(OfferDO::getOrderId, event.getOrderId()));
    }
}
