package cn.iocoder.yudao.module.sale.listener;

import cn.iocoder.yudao.framework.apollo.core.event.prod.ProductLineUpdateAutoProcessOverWeightExceptionEvent;
import cn.iocoder.yudao.framework.apollo.core.event.prod.ProductUpdateAutoProcessOverWeightExceptionEvent;
import cn.iocoder.yudao.module.order.service.orderException.OrderExceptionService;
import lombok.AllArgsConstructor;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;

/**
 * 异常自动处理监听
 *
 * @author yanghao
 */
@Component("orderExceptionAutoProcessListener")
@AllArgsConstructor
public class OrderExceptionProcessListener {

    private final OrderExceptionService orderExceptionService;

    /**
     * 产品更新导致需要自动处理异常
     *
     * @param event
     */
    @EventListener(ProductUpdateAutoProcessOverWeightExceptionEvent.class)
    public void productUpdateAutoProcessException(ProductUpdateAutoProcessOverWeightExceptionEvent event) {

        Long prodId = event.getProdId();
        BigDecimal airWeightLimitOld = event.getAirWeightLimitOld();
        BigDecimal airWeightLimitNew = event.getAirWeightLimitNew();
        // 1、老值不为空，新值为空时需要自动处理未处理的未达标的商品超重异常  2、老值不为空，新值大于老值时，需要自动处理未处理的未达标的商品超重异常
        if (airWeightLimitOld != null && (airWeightLimitNew == null || airWeightLimitNew.compareTo(airWeightLimitOld) == 1)) {
            orderExceptionService.autoProcessGoodsWeightException(prodId, airWeightLimitNew, null);
        }

    }

    /**
     * 产品价格更新导致需要自动处理异常
     *
     * @param event
     */
    @EventListener(ProductLineUpdateAutoProcessOverWeightExceptionEvent.class)
    public void productLineUpdateAutoProcessOverWeightExceptionEvent(ProductLineUpdateAutoProcessOverWeightExceptionEvent event) {

        Long prodId = event.getProdId();
        Long lineId = event.getLineId();
        Long channelId = event.getChannelId();
        BigDecimal weightLimitOld = event.getLineWeightLimitOld();
        BigDecimal weightLimitNew = event.getLineWeightLimitNew();
        // 1、老值不为空，新值为空时需要自动处理未处理的未达标的商品超重异常  2、老值不为空，新值大于老值时，需要自动处理未处理的未达标的商品超重异常
        if (weightLimitOld != null && (weightLimitNew == null || weightLimitNew.compareTo(weightLimitOld) == 1)) {
            // orderExceptionService.autoProcessGoodsWeightException(prodId, weightLimitNew, null);
            orderExceptionService.autoProcessGoodsLineWeightException(lineId, channelId,prodId, weightLimitNew, null);
        }

    }


}
