package cn.iocoder.yudao.module.sale.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.apollo.core.dto.ChannelClearancePriceDto;
import cn.iocoder.yudao.framework.apollo.core.dto.ChannelDto;
import cn.iocoder.yudao.framework.apollo.core.event.ChannelPriceEvent;
import cn.iocoder.yudao.framework.apollo.core.event.LineChannelUpdateAirWeightEvent;
import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import cn.iocoder.yudao.framework.apollo.core.event.ChannelDataEvent;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPackaging.LineChannelPackagingDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.lineChannelPriceStepClearance.LineChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.depository.service.lineChannelPackaging.LineChannelPackagingService;
import cn.iocoder.yudao.module.depository.service.lineChannelPriceStepClearance.LineChannelPriceStepClearanceService;
import cn.iocoder.yudao.module.depository.vo.lineChannelPriceStepClearance.LineChannelPriceStepClearanceQueryVO;
import cn.iocoder.yudao.module.sale.convert.channel.ChannelConvert;
import cn.iocoder.yudao.module.sale.dal.dataobject.channel.ChannelDO;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPackaging.ChannelPackagingDO;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPriceStepClearance.ChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.sale.service.channel.ChannelService;
import cn.iocoder.yudao.module.sale.service.channelPackaging.ChannelPackagingService;
import cn.iocoder.yudao.module.sale.service.channelPriceStepClearance.ChannelPriceStepClearanceService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 查询渠道详情监听
 *
 * @author zhengYi
 */
@Component("QueryChannelInfoListener")
@AllArgsConstructor
public class QueryChannelInfoListener {

    private final ChannelService channelService;

    private final ChannelPackagingService channelPackagingService;


    private final ChannelPriceStepClearanceService channelPriceStepClearanceService;



    private final LineChannelPackagingService lineChannelPackagingService;

    private final LineChannelPriceStepClearanceService lineChannelPriceStepClearanceService;

    /**
     * 根据渠道id查询渠道详情
     *
     * @param event 渠道信息
     */
    @EventListener(QueryChannelInfoEvent.class)
    public void queryChannelInfo(QueryChannelInfoEvent event) {
        ChannelDO channelDO = channelService.getChannel(event.getChannelId());
        if (Objects.nonNull(channelDO)) {
            BeanUtils.copyProperties(channelDO, event);
        }
    }

    @EventListener(ChannelDataEvent.class)
    public void queryChannelData(ChannelDataEvent event) {
        List<ChannelDO> channelList = channelService.list();
        List<Long> channelIdList = channelList.stream()
                .map(ChannelDO::getChannelId)
                .collect(Collectors.toList());
        event.setChannelIdList(channelIdList);

        List<ChannelDto> channelDtos = ChannelConvert.INSTANCE.convertDtoList(channelList);
        event.setChannelDtoList(channelDtos);
    }

    @EventListener(ChannelPriceEvent.class)
    public void queryChannelPrice(ChannelPriceEvent event) {
        //找到包装
        List<ChannelPackagingDO> packagingDOList = new ArrayList<>();
        if (StringUtils.isNotBlank(event.getPackingType())) {
            packagingDOList = channelPackagingService.list(new LambdaQueryWrapper<ChannelPackagingDO>()
                    .eq(ChannelPackagingDO::getChannelId, event.getChannelId())
                    .apply(event.getPackingType() != null, "FIND_IN_SET('" + event.getPackingType() + "',packaging_types)")
            );
        } else if (CollectionUtil.isNotEmpty(event.getPackingTypes())) {
            for (String packingType : event.getPackingTypes()) {
                List<ChannelPackagingDO> packagingList = channelPackagingService.list(new LambdaQueryWrapper<ChannelPackagingDO>()
                        .eq(ChannelPackagingDO::getChannelId, event.getChannelId())
                        .apply(event.getPackingType() != null, "FIND_IN_SET('" + packingType + "',packaging_types)")
                );
                if (CollectionUtil.isNotEmpty(packagingList)) {
                    packagingDOList.addAll(packagingList);
                }
            }

        } else {
            packagingDOList = channelPackagingService.list(new LambdaQueryWrapper<ChannelPackagingDO>()
                    .eq(ChannelPackagingDO::getChannelId, event.getChannelId())
            );
        }

        if (CollectionUtil.isNotEmpty(packagingDOList)) {
            List<ChannelPriceStepClearanceDO> allChannelPriceStepClearanceDOList = new ArrayList<>();
            for (ChannelPackagingDO channelPackagingDO : packagingDOList) {
                //多个包装类型
                BigDecimal airWeightLimit = channelPackagingDO.getAirWeightLimit();
                event.setAirWeightLimit(airWeightLimit);
                List<Long> packIdList = packagingDOList.stream()
                        .map(ChannelPackagingDO::getId)
                        .collect(Collectors.toList());
                List<ChannelPriceStepClearanceDO> channelPriceStepClearanceDOList = channelPriceStepClearanceService.list(new LambdaQueryWrapper<ChannelPriceStepClearanceDO>()
                        .in(ChannelPriceStepClearanceDO::getPackagingId, packIdList)
                );
                if (CollectionUtil.isNotEmpty(channelPriceStepClearanceDOList)){
                    allChannelPriceStepClearanceDOList.addAll(channelPriceStepClearanceDOList);
                }
            }
            List<ChannelClearancePriceDto> channelClearancePriceDtoList = new ArrayList<>();
//            BeanUtils.copyProperties(channelPriceStepClearanceDOList, channelClearancePriceDtoList);
            for (ChannelPriceStepClearanceDO source : allChannelPriceStepClearanceDOList) {
                ChannelClearancePriceDto target = new ChannelClearancePriceDto();
                BeanUtils.copyProperties(source, target);
                channelClearancePriceDtoList.add(target);
            }

            event.setChannelClearancePriceDtoList(channelClearancePriceDtoList);
        }
    }




    @EventListener(LineChannelUpdateAirWeightEvent.class)
    public void updateLineChannelAirWeightEvent(LineChannelUpdateAirWeightEvent event) {
        Long lineId = event.getLineId();
        Long channelId = event.getChannelId();
        String packingType = event.getPackingType();
        BigDecimal airWeightLimit = event.getAirWeightLimit();

        List<LineChannelPackagingDO> lineChannelPackagingDOList = lineChannelPackagingService.list(new LambdaQueryWrapper<LineChannelPackagingDO>()
                .eq(LineChannelPackagingDO::getLineId, lineId)
                .apply(channelId != null, "FIND_IN_SET('" + channelId + "', channel_ids)")
                .apply(packingType != null, "FIND_IN_SET('" + packingType + "', packaging_types)"));
        if (CollectionUtil.isEmpty(lineChannelPackagingDOList)) return;

        LineChannelPackagingDO lineChannelPackagingDO = lineChannelPackagingDOList.stream()
                .filter(t -> t.getChannelIds().equals(channelId.toString()) && t.getPackagingTypes().equals(packingType))
                .findFirst().orElse(null);
        if (lineChannelPackagingDO != null) {
            // 这说明是单独配置的一条数据，不跟其他渠道和包装类型合用，故直接设置新的超重值即可
            lineChannelPackagingDO.setAirWeightLimit(airWeightLimit);
            lineChannelPackagingService.updateById(lineChannelPackagingDO);
        } else {
            // 没有单独配置一条，说明是与其他渠道或包装合用的，故需要拆分出来。
            // 原有的可能是这样的：渠道1,2,3，包装类型为a,b,c 的1条数据 需要拆分成 渠道1,2和包装a,b（原数据）  和  渠道3和包装c 的一条新数据,
            // 原数据不更新重量上限值。新增的数据更新上限值。

            lineChannelPackagingDO = lineChannelPackagingDOList.get(0);

            // 可能的类型: 渠道1,2,3+包装a,b,c（原数据修改为1,2+a,b 新增3+c）  1,2,3+c（原数据修改为1,2+c 新增3+c）   3+a,b,c;(原数据修改为3+a,b 新增3+c)   3+c(在if中已处理)
            // 综上分析  新增3+c是每个类型都必须做的， 然后就是渠道条件中只有3的情况下不动，包含3时需要删除。 包装只有c时不动，包含c时需要删除。 然后更新原数据。
            if (!lineChannelPackagingDO.getChannelIds().equals(channelId.toString())) {
                String channelIdsNew = Arrays.stream(lineChannelPackagingDO.getChannelIds().split(",")).filter(t -> !t.equals(channelId)).collect(Collectors.joining(","));
                lineChannelPackagingDO.setChannelIds(channelIdsNew);
            }
            if (!lineChannelPackagingDO.getPackagingTypes().equals(packingType)) {
                String packagingTypeNew = Arrays.stream(lineChannelPackagingDO.getPackagingTypes().split(",")).filter(t -> !t.equals(packingType)).collect(Collectors.joining(","));
                lineChannelPackagingDO.setPackagingTypes(packagingTypeNew);
            }
            lineChannelPackagingService.updateById(lineChannelPackagingDO);

            //新增一个渠道包装类型
            LineChannelPackagingDO lineChannelPackagingDONew = BeanUtil.copyProperties(lineChannelPackagingDO, LineChannelPackagingDO.class);
            lineChannelPackagingDONew.setId(null);
            lineChannelPackagingDONew.setLineId(lineId);
            lineChannelPackagingDONew.setChannelIds(channelId.toString());
            lineChannelPackagingDONew.setPackagingTypes(packingType);
            lineChannelPackagingDONew.setAirWeightLimit(airWeightLimit);
            lineChannelPackagingService.save(lineChannelPackagingDONew);

            // 新增渠道包装对应的清关费
            LineChannelPriceStepClearanceQueryVO paramQueryVO = new LineChannelPriceStepClearanceQueryVO();
            paramQueryVO.setLineId(lineId);
            paramQueryVO.setPackagingId(lineChannelPackagingDO.getId());
            List<LineChannelPriceStepClearanceDO> lineChannelPriceStepClearanceList = lineChannelPriceStepClearanceService.getLineChannelPriceStepClearanceList(paramQueryVO);

            if (CollectionUtil.isNotEmpty(lineChannelPriceStepClearanceList)) {
                List<LineChannelPriceStepClearanceDO> lineChannelPriceStepClearanceDOList = lineChannelPriceStepClearanceList.stream()
                        .map(lineChannelPriceStepClearanceDO -> {
                            LineChannelPriceStepClearanceDO channelPriceStepClearanceDO1 = BeanUtil.copyProperties(lineChannelPriceStepClearanceDO, LineChannelPriceStepClearanceDO.class);
                            channelPriceStepClearanceDO1.setId(null);
                            channelPriceStepClearanceDO1.setLineId(lineId);
                            channelPriceStepClearanceDO1.setPackagingId(lineChannelPackagingDONew.getId());
                            return channelPriceStepClearanceDO1;
                        })
                        .collect(Collectors.toList());
                lineChannelPriceStepClearanceService.saveBatch(lineChannelPriceStepClearanceDOList);
            }

        }

    }



}
