package cn.iocoder.yudao.module.sale.listener;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.apollo.core.event.QueryEstimateEnterOpenSeaTimeEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.ecw.enums.CustomerStatusEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.vo.order.OrderBackPageVO;
import cn.iocoder.yudao.module.order.vo.order.OrderQueryVO;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.module.sale.service.offer.OfferService;
import cn.iocoder.yudao.module.sale.vo.offer.OfferExportReqVO;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 编号记录监听
 *
 * @author zhengYi
 */
@Component("QueryEstimateEnterOpenSeaTimeListener")
@AllArgsConstructor
public class QueryEstimateEnterOpenSeaTimeListener {

    private final OrderQueryService orderQueryService;

    private final OfferService offerService;

    private final CustomerService customerService;

    /**
     * 根据订单查询
     *
     * @param event 编号记录
     */
    @EventListener(QueryEstimateEnterOpenSeaTimeEvent.class)
    public void QueryEstimateEnterOpenSeaTime(QueryEstimateEnterOpenSeaTimeEvent event) {

        Long customerId = event.getCustomerId();
        Date customerCreateTime = event.getCustomerCreateTime();
        String transportType = event.getTransportType();

        if (StringUtils.isEmpty(transportType)) {

            //先查找客户，若是空运，直接设置为成交
            CustomerDO customer = customerService.getById(customerId);
            transportType = customer.getTransportType();
        }
        if (StringUtils.isNotEmpty(transportType) && (transportType.contains("3"))) {
            //海运整柜客户不入，故入公海时间要设置为空
            event.setReturnEstimateEnterOpenSeaTime(null);
            //海运整柜客户状态 必要为成交
            event.setReturnEstimateCustomerStatus(CustomerStatusEnum.DEAL.getValue());
            return;
        }



        // 没有订单没有报价，则设置其为线索，且预计进入公海池时间为 客户创建时间1个月后
        Date estimateEnterOpenSeaTime = DateUtil.offsetMonth(customerCreateTime, 1).toJdkDate();
        int customerStatus = CustomerStatusEnum.CLUE.getValue();

        // 先查订单
        OrderQueryVO query = new OrderQueryVO();
        query.setCustomerDetailId(customerId);
        PageVO page = new PageVO();
        page.setPage(1);
        page.setRows(100);
        PageResult<OrderBackPageVO> orderBackPageVOPageResult = orderQueryService.orderPage(query, page);
        if (orderBackPageVOPageResult != null && CollectionUtil.isNotEmpty(orderBackPageVOPageResult.getList())) {
            List<OrderBackPageVO> orderBackPageVOList = orderBackPageVOPageResult.getList();

            //状态大于5已入仓的数据
            Date rucangTime = orderBackPageVOList.stream().filter(t -> t.getStatus() >= OrderStatusEnum.IN_WAREHOUSE.getValue() && t.getRucangTime() != null)
                    .map(OrderBackPageVO::getRucangTime).max(Date::compareTo).orElse(null);
            //若还是入仓中的，则更新为意向
            //状态包含 2 已下单  3 入仓中  4特价申请
            Date createTime = orderBackPageVOList.stream().filter(t -> t.getStatus() >= OrderStatusEnum.ORDER_PLACED.getValue() && t.getStatus() < OrderStatusEnum.IN_WAREHOUSE.getValue())
                    .map(OrderBackPageVO::getCreateTime).max(Date::compareTo).orElse(null);

            if (rucangTime != null && createTime == null) {

                estimateEnterOpenSeaTime = DateUtil.offsetMonth(rucangTime, 6).toJdkDate();
                customerStatus = CustomerStatusEnum.DEAL.getValue();

            } else if (rucangTime != null && createTime != null) {

                // 入仓时间大于未入仓的创建订单时间，则使用入仓时间+6, 否则使用订单创建时间+3
                estimateEnterOpenSeaTime = rucangTime.compareTo(createTime) == 1 ?
                        DateUtil.offsetMonth(rucangTime, 6).toJdkDate()
                        :
                        DateUtil.offsetMonth(createTime, 3).toJdkDate();
                customerStatus = CustomerStatusEnum.DEAL.getValue();

            } else if (rucangTime == null && createTime != null) {
                //只有未入仓订单，意向
                estimateEnterOpenSeaTime = DateUtil.offsetMonth(createTime, 3).toJdkDate();
                customerStatus = CustomerStatusEnum.INTENTION.getValue();
            }
        }

        //没有订单 查报价
        else {


            List<OfferDO> offerDOList = offerService.getOfferList(new OfferExportReqVO().setRelationId(customerId));
            if (CollectionUtil.isNotEmpty(offerDOList)) {

                Date maxCreateTime = offerDOList.stream().map(BaseDO::getCreateTime).max(Date::compareTo).get();
                estimateEnterOpenSeaTime = DateUtil.offsetMonth(maxCreateTime, 1).toJdkDate();

                customerStatus = CustomerStatusEnum.INTENTION.getValue();
            }
        }

        event.setReturnEstimateEnterOpenSeaTime(estimateEnterOpenSeaTime);
        event.setReturnEstimateCustomerStatus(customerStatus);
    }
}
