package cn.iocoder.yudao.module.sale.service.channel;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.sale.vo.channel.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.channel.ChannelDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 渠道管理 Service 接口
 *
 * @author 系统管理员
 */
public interface ChannelService extends IService<ChannelDO> {

    /**
     * 创建渠道管理
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createChannel(@Valid ChannelCreateReqVO createReqVO);

    /**
     * 更新渠道管理
     *
     * @param updateReqVO 更新信息
     */
    void updateChannel(@Valid ChannelUpdateReqVO updateReqVO);

    /**
     * 更新渠道状态
     *
     * @param channelId 渠道ID
     */
    void updateChannelStatus(Long channelId);

    /**
     * 删除渠道管理
     *
     * @param channelId 编号
     */
    void deleteChannel(Long channelId);

    /**
     * 获得渠道管理
     *
     * @param channelId 编号
     * @return 渠道管理
     */
    ChannelDO getChannel(Long channelId);

    /**
     * 获得渠道管理列表
     *
     * @param ids 编号
     * @return 渠道管理列表
     */
    List<ChannelDO> getChannelList(Collection<Long> ids);

    /**
     * 获得渠道管理分页
     *
     * @param pageReqVO 分页查询
     * @return 渠道管理分页
     */
    PageResult<ChannelDO> getChannelPage(ChannelPageReqVO pageReqVO);

    /**
     * 获得渠道管理列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 渠道管理列表
     */
    List<ChannelDO> getChannelList(ChannelExportReqVO exportReqVO);

    ChannelRespVO getInfo(Long channelId);

    void updateChannelCost(ChannelCostUpdateReqVO costUpdateReqVO);

    /**
     * 获得渠道管理分页
     *
     * @param pageVO 分页查询
     * @return 渠道管理分页
     */
    PageResult<ChannelRespVO> channelPage(ChannelPageReqVO pageVO);

    List<ChannelDO> getChannelSimpleList();
}
