package cn.iocoder.yudao.module.sale.service.channel.impl;

import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.order.service.order.OrderAirService;
import cn.iocoder.yudao.module.order.service.orderException.OrderExceptionService;
import cn.iocoder.yudao.module.product.service.product.ProductPriceService;
import cn.iocoder.yudao.module.product.service.productPriceSnapshot.ProductPriceSnapshotService;
import cn.iocoder.yudao.module.sale.convert.channel.ChannelConvert;
import cn.iocoder.yudao.module.sale.dal.dataobject.channel.ChannelDO;
import cn.iocoder.yudao.module.sale.dal.mysql.channel.ChannelMapper;
import cn.iocoder.yudao.module.sale.dal.mysql.channelPackaging.ChannelPackagingMapper;
import cn.iocoder.yudao.module.sale.dal.mysql.channelPriceStepClearance.ChannelPriceStepClearanceMapper;
import cn.iocoder.yudao.module.sale.service.channel.ChannelService;
import cn.iocoder.yudao.module.sale.service.channelPackaging.ChannelPackagingService;
import cn.iocoder.yudao.module.sale.service.channelPriceStepClearance.ChannelPriceStepClearanceService;
import cn.iocoder.yudao.module.sale.vo.channel.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 渠道管理 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ChannelServiceImpl extends AbstractService<ChannelMapper, ChannelDO> implements ChannelService {

    @Autowired
    private ChannelMapper channelMapper;

    @Autowired
    private ChannelPackagingMapper channelPackagingMapper;

    @Autowired
    private ChannelPackagingService channelPackagingService;
    @Autowired
    private ChannelPriceStepClearanceService channelPriceStepClearanceService;
    @Autowired
    private ChannelPriceStepClearanceMapper channelPriceStepClearanceMapper;

    @Autowired
    private ProductPriceSnapshotService productPriceSnapshotService;

    @Autowired
    private ProductPriceService productPriceService;

    @Autowired
    private OrderAirService orderAirService;

    @Autowired
    private OrderExceptionService orderExceptionService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createChannel(ChannelCreateReqVO createReqVO) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();

        // 插入渠道信息
        ChannelDO channel = ChannelConvert.INSTANCE.convert(createReqVO);
        Date now = new Date();
        channel.setCreator(String.valueOf(userId));
        channel.setUpdater(String.valueOf(userId));
        channel.setCreateTime(now);
        channel.setUpdateTime(now);
        channelMapper.insert(channel);

        // 返回
        Long channelId = channel.getChannelId();
        return channelId;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateChannel(ChannelUpdateReqVO updateReqVO) {

        Long userId = SecurityFrameworkUtils.getLoginUserId();

        Long channelId = updateReqVO.getChannelId();
        ChannelDO channnelDO = channelMapper.selectById(channelId);
        // 校验存在
        if (channnelDO == null) {
            throw exception(CHANNEL_NOT_EXISTS);
        }
        // 校验渠道可出特性是否有修改
        boolean isChangeArrId = Boolean.FALSE;
        if (!StringUtils.equals(channnelDO.getAttrId() == null ? "" : channnelDO.getAttrId(), updateReqVO.getAttrId() == null ? "" : updateReqVO.getAttrId())) {
            // 可出特效变动，需要检查使用了该渠道的所有入仓订单，是否需要自动处理不可出渠道异常
            isChangeArrId = Boolean.TRUE;
        }
        if (channnelDO.getCountryId() > 0 && !Objects.equals(updateReqVO.getCountryId(), channnelDO.getCountryId())) {
            // 渠道已经设置了目的国，否则不校验
            // 当渠道所属目的国变动时，校验该渠道是否设置了线路价格，如果有，则不允许修改
            // 存在路线价格，不允许修改目的国
            Long productPriceCount = productPriceService.channelPriceCount(updateReqVO.getChannelId());
            if (productPriceCount > 0) {
                throw exception(CHANNEL_HAVE_PRODUCT_PRICE_CAN_NOT_UPDATE_COUNTRY);
            }
        }

        // 更新
        ChannelDO updateObj = ChannelConvert.INSTANCE.convert(updateReqVO);
        updateObj.setUpdateTime(new Date());
        updateObj.setUpdater(String.valueOf(userId));
        channelMapper.updateById(updateObj);

        // 更新空运清关费阶梯价格
        channelPackagingMapper.physicsDelete(channelId);
        channelPriceStepClearanceMapper.physicsDelete(channelId);

        if (isChangeArrId) {
            // 渠道可出特效变动，检查所有使用了改渠道的已入仓订单渠道不可出异常情况
            QueryChannelInfoEvent event = new QueryChannelInfoEvent();
            BeanUtils.copyProperties(updateObj, event);
            orderAirService.inspectWarehouseInOrderChannelNoCan(event);
        }
    }

    @Override
    public void updateChannelStatus(Long channelId) {
        channelMapper.updateChannelStatus(channelId);
    }

    @Override
    public void deleteChannel(Long id) {
        // 校验存在
        this.validateChannelExists(id);

        // 存在路线价格 不可删除
        Long productPriceCount = productPriceService.channelPriceCount(id);
        if (productPriceCount > 0) {
            throw exception(CHANNEL_HAVE_PRODUCT_PRICE_CAN_NOT_DELETE);
        }

        // 删除包装类型
        this.update(new LambdaUpdateWrapper<ChannelDO>()
                .set(ChannelDO::getDeleted, Boolean.TRUE)
                .eq(ChannelDO::getChannelId, id));
    }

    private void validateChannelExists(Long id) {
        if (channelMapper.selectById(id) == null) {
            throw exception(CHANNEL_NOT_EXISTS);
        }
    }

    @Override
    public ChannelDO getChannel(Long id) {
        return channelMapper.selectById(id);
    }

    @Override
    public List<ChannelDO> getChannelList(Collection<Long> ids) {
        return channelMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ChannelDO> getChannelPage(ChannelPageReqVO pageReqVO) {

        return channelMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ChannelDO> getChannelList(ChannelExportReqVO exportReqVO) {
        return channelMapper.selectList(exportReqVO);
    }

    @Override
    public ChannelRespVO getInfo(Long channelId) {
        ChannelDO channelDO = channelMapper.selectById(channelId);
        if (Objects.isNull(channelDO)) {
            throw exception(CHANNEL_NOT_EXISTS);
        }
        ChannelRespVO channelRespVO = ChannelConvert.INSTANCE.convert(channelDO);

        return channelRespVO;
    }

    @Override
    public void updateChannelCost(ChannelCostUpdateReqVO costUpdateReqVO) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        ChannelDO channelDO = channelMapper.selectById(costUpdateReqVO.getChannelId());
        if (channelDO == null) {
            throw exception(CHANNEL_NOT_EXISTS);
        }
        BigDecimal customsClearUnit = channelDO.getCustomsClearUnit().add(costUpdateReqVO.getModifyCustomsClearUnit());

        BigDecimal weightUnitPrice = costUpdateReqVO.getWeightUnitPrice();
        if (customsClearUnit.compareTo(BigDecimal.ZERO) < 0 || weightUnitPrice.compareTo(BigDecimal.ZERO) < 0) {
            throw exception(CHANNEL_PRICE_NOT_NEGATIVE);
        }
        channelDO.setCustomsClearUnit(customsClearUnit);
        channelDO.setWeightUnitPrice(weightUnitPrice);
        channelMapper.updateById(channelDO);

    }

    @Override
    public PageResult<ChannelRespVO> channelPage(ChannelPageReqVO pageVO) {
        IPage<ChannelRespVO> mpPage = MyBatisUtils.buildPage(pageVO);
        pageVO.setPageNo((int) ((mpPage.getCurrent() - 1) * mpPage.getSize()));
        pageVO.setPageSize((int) (mpPage.getSize()));
        pageVO.setLang(I18nMessage.getLang());
        Long total = channelMapper.countChannelList(pageVO);
        log.warn(I18nMessage.getLang().toString());
        List<ChannelRespVO> list = channelMapper.channelPage(pageVO, I18nMessage.getLang());
        // 渠道页面已经不选归属仓库了
        // list = list.stream().peek(vo -> vo.setWarehouseNameList(warehouseMapper.findWarehouseNameByIds(Arrays.asList(vo.getWarehouseIds().split(StrUtil.COMMA)), I18nMessage.getLang()))).collect(Collectors.toList());
        PageResult<ChannelRespVO> pageResult = new PageResult<>(list, total, mpPage.getSize(), mpPage.getCurrent(), (total + mpPage.getSize() - 1) / mpPage.getSize());

        // 设置包装价格数量。先按channelId集合查询出所有阶梯价格数据，再按渠道查询对应的包装数量。
        List<ChannelRespVO> channelList = pageResult.getList();
        return pageResult;
    }

    @Override
    public List<ChannelDO> getChannelSimpleList() {
        return channelMapper.selectList(new LambdaQueryWrapper<ChannelDO>());
    }
}
