package cn.iocoder.yudao.module.sale.service.channelPackaging;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.sale.vo.channelPackaging.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPackaging.ChannelPackagingDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 渠道清关费包装类型 Service 接口
 *
 * @author yanghao
 */
public interface ChannelPackagingService extends IService<ChannelPackagingDO> {

    /**
     * 创建渠道清关费包装类型
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createChannelPackaging(@Valid ChannelPackagingCreateReqVO createReqVO);

    /**
     * 更新渠道清关费包装类型
     * @param updateReqVO 更新信息
     */
    void updateChannelPackaging(@Valid ChannelPackagingUpdateReqVO updateReqVO);

    /**
     * 删除渠道清关费包装类型
     * @param id 编号
     */
    void deleteChannelPackaging(Long id);

    /**
     * 根据渠道id删除渠道清关费包装类型
     * @param channelId 渠道编号
     */
    void deleteChannelPackagingByChannelId(Long channelId);

    /**
     * 获得渠道清关费包装类型
     * @param id 编号
     * @return 渠道清关费包装类型
     */
    ChannelPackagingDO getChannelPackaging(Long id);



    /**
     * 获得渠道清关费包装类型列表
     * @param channelId 渠道id
     * @return 渠道清关费包装类型列表
     */
    List<ChannelPackagingDO> getChannelPackagingWithPriceStepClearanceList(Long channelId);



    /**
     * 获得渠道清关费包装类型列表
     * @param ids 编号
     * @return 渠道清关费包装类型列表
     */
    List<ChannelPackagingDO> getChannelPackagingWithPriceStepClearanceList(Collection<Long> ids);

    /**
     * 获得渠道清关费包装类型分页
     * @param page 分页查询
     * @param query 查询
     * @return 渠道清关费包装类型分页
     */
    PageResult<ChannelPackagingDO> getChannelPackagingPage(ChannelPackagingQueryVO query, PageVO page);

    /**
     * 获得渠道清关费包装类型列表, 用于 Excel 导出
     * @param query 查询
     * @return 渠道清关费包装类型列表
     */
    List<ChannelPackagingDO> getChannelPackagingWithPriceStepClearanceList(ChannelPackagingQueryVO query);
}
