package cn.iocoder.yudao.module.sale.service.channelPackaging;

import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPriceStepClearance.ChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.sale.dal.mysql.channelPriceStepClearance.ChannelPriceStepClearanceMapper;
import cn.iocoder.yudao.module.sale.service.channelPriceStepClearance.ChannelPriceStepClearanceService;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.sale.vo.channelPackaging.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPackaging.ChannelPackagingDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.sale.convert.channelPackaging.ChannelPackagingConvert;
import cn.iocoder.yudao.module.sale.dal.mysql.channelPackaging.ChannelPackagingMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.sale.enums.ErrorCodeConstants.*;

/**
 * 渠道清关费包装类型 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class ChannelPackagingServiceImpl extends AbstractService<ChannelPackagingMapper, ChannelPackagingDO> implements ChannelPackagingService {

    @Resource
    private ChannelPackagingMapper channelPackagingMapper;

    @Resource
    private ChannelPriceStepClearanceService channelPriceStepClearanceService;

    @Override
    public Long createChannelPackaging(ChannelPackagingCreateReqVO createReqVO) {
        // 插入
        ChannelPackagingDO channelPackaging = ChannelPackagingConvert.INSTANCE.convert(createReqVO);
        channelPackagingMapper.insert(channelPackaging);
        // 返回
        return channelPackaging.getId();
    }

    @Override
    public void updateChannelPackaging(ChannelPackagingUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateChannelPackagingExists(updateReqVO.getId());
        // 更新
        ChannelPackagingDO updateObj = ChannelPackagingConvert.INSTANCE.convert(updateReqVO);
        channelPackagingMapper.updateById(updateObj);
    }

    @Override
    public void deleteChannelPackaging(Long id) {
        // 校验存在
        this.validateChannelPackagingExists(id);
        // 删除
        channelPackagingMapper.deleteById(id);
    }

    @Override
    public void deleteChannelPackagingByChannelId(Long channelId) {
        this.update(new LambdaUpdateWrapper<ChannelPackagingDO>()
                .set(ChannelPackagingDO::getDeleted, Boolean.TRUE)
                .eq(ChannelPackagingDO::getChannelId, channelId));

        // 删除清关费阶梯价格
        channelPriceStepClearanceService.update(new LambdaUpdateWrapper<ChannelPriceStepClearanceDO>()
                .set(ChannelPriceStepClearanceDO::getDeleted, Boolean.TRUE)
                .eq(ChannelPriceStepClearanceDO::getChannelId, channelId));
    }

    private void validateChannelPackagingExists(Long id) {
        if (channelPackagingMapper.selectById(id) == null) {
            throw exception(CHANNEL_PACKAGING_NOT_EXISTS);
        }
    }

    @Override
    public ChannelPackagingDO getChannelPackaging(Long id) {
        return channelPackagingMapper.selectById(id);
    }

    @Override
    public List<ChannelPackagingDO> getChannelPackagingWithPriceStepClearanceList(Long channelId) {
        List<ChannelPackagingDO> channelPackagingDOList = channelPackagingMapper.selectList(
                new LambdaQueryWrapperX<ChannelPackagingDO>().eq(ChannelPackagingDO::getChannelId, channelId));
        if (CollectionUtil.isNotEmpty(channelPackagingDOList)) {
            for (ChannelPackagingDO channelPackagingDO : channelPackagingDOList) {
                List<ChannelPriceStepClearanceDO> channelPriceStepClearanceList = channelPriceStepClearanceService.getChannelPriceStepClearanceList(channelPackagingDO.getId());
                channelPackagingDO.setChannelPriceStepClearanceDOList(channelPriceStepClearanceList);
            }
        }

        return channelPackagingDOList;
    }

    @Override
    public List<ChannelPackagingDO> getChannelPackagingWithPriceStepClearanceList(Collection<Long> ids) {
        return channelPackagingMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ChannelPackagingDO> getChannelPackagingPage(ChannelPackagingQueryVO query, PageVO page) {
        return channelPackagingMapper.selectPage(page, query);
    }

    @Override
    public List<ChannelPackagingDO> getChannelPackagingWithPriceStepClearanceList(ChannelPackagingQueryVO query) {
        return channelPackagingMapper.selectList(query);
    }
}
