package cn.iocoder.yudao.module.sale.service.channelPriceStepClearance;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.sale.vo.channelPriceStepClearance.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPriceStepClearance.ChannelPriceStepClearanceDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.sale.convert.channelPriceStepClearance.ChannelPriceStepClearanceConvert;
import cn.iocoder.yudao.module.sale.dal.mysql.channelPriceStepClearance.ChannelPriceStepClearanceMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.sale.enums.ErrorCodeConstants.*;

/**
 * 渠道清关费阶梯价格 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class ChannelPriceStepClearanceServiceImpl extends AbstractService<ChannelPriceStepClearanceMapper, ChannelPriceStepClearanceDO> implements ChannelPriceStepClearanceService {

    @Resource
    private ChannelPriceStepClearanceMapper channelPriceStepClearanceMapper;

    @Override
    public Long createChannelPriceStepClearance(ChannelPriceStepClearanceCreateReqVO createReqVO) {
        // 插入
        ChannelPriceStepClearanceDO channelPriceStepClearance = ChannelPriceStepClearanceConvert.INSTANCE.convert(createReqVO);
        channelPriceStepClearanceMapper.insert(channelPriceStepClearance);
        // 返回
        return channelPriceStepClearance.getId();
    }

    @Override
    public void updateChannelPriceStepClearance(ChannelPriceStepClearanceUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateChannelPriceStepClearanceExists(updateReqVO.getId());
        // 更新
        ChannelPriceStepClearanceDO updateObj = ChannelPriceStepClearanceConvert.INSTANCE.convert(updateReqVO);
        channelPriceStepClearanceMapper.updateById(updateObj);
    }

    @Override
    public void deleteChannelPriceStepClearance(Long id) {
        // 校验存在
        this.validateChannelPriceStepClearanceExists(id);
        // 删除
        channelPriceStepClearanceMapper.deleteById(id);
    }

    private void validateChannelPriceStepClearanceExists(Long id) {
        if (channelPriceStepClearanceMapper.selectById(id) == null) {
            throw exception(CHANNEL_PRICE_STEP_CLEARANCE_NOT_EXISTS);
        }
    }

    @Override
    public ChannelPriceStepClearanceDO getChannelPriceStepClearance(Long id) {
        return channelPriceStepClearanceMapper.selectById(id);
    }

    @Override
    public List<ChannelPriceStepClearanceDO> getChannelPriceStepClearanceList(Collection<Long> ids) {
        return channelPriceStepClearanceMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ChannelPriceStepClearanceDO> getChannelPriceStepClearancePage(ChannelPriceStepClearanceQueryVO query, PageVO page) {
        return channelPriceStepClearanceMapper.selectPage(page, query);
    }

    @Override
    public List<ChannelPriceStepClearanceDO> getChannelPriceStepClearanceList(ChannelPriceStepClearanceQueryVO query) {
        return channelPriceStepClearanceMapper.selectList(query);
    }

    @Override
    public List<ChannelPriceStepClearanceDO> getChannelPriceStepClearanceList(Long packagingId) {
        return channelPriceStepClearanceMapper.selectList(new LambdaQueryWrapperX<ChannelPriceStepClearanceDO>()
                .eq(ChannelPriceStepClearanceDO::getPackagingId, packagingId)
                .eq(BaseDO::getDeleted, false)
                .orderByAsc(ChannelPriceStepClearanceDO::getStartNum));
    }
}
