package cn.iocoder.yudao.module.sale.service.deptTarget;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.PersonalDeptTargetDO;
import cn.iocoder.yudao.module.sale.dto.AchievementStatisticsDto;
import cn.iocoder.yudao.module.sale.dto.MyAchievementDto;
import cn.iocoder.yudao.module.sale.dto.MyAchievementRetDto;
import cn.iocoder.yudao.module.sale.vo.deptTarget.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.DeptTargetDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 部门业绩目标设置 Service 接口
 *
 * @author 系统管理员
 */
public interface DeptTargetService extends IService<DeptTargetDO> {

    /**
     * 创建部门业绩目标设置
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDeptTarget(@Valid DeptTargetCreateReqVO createReqVO);

    /**
     * 更新部门业绩目标设置
     * @param updateReqVO 更新信息
     */
    void updateDeptTarget(@Valid DeptTargetUpdateReqVO updateReqVO);

    /**
     * 删除部门业绩目标设置
     * @param id 编号
     */
    void deleteDeptTarget(Long id);

    /**
     * 获得部门业绩目标设置
     * @param id 编号
     * @return 部门业绩目标设置
     */
    DeptTargetDO getDeptTarget(Long id);

    /**
     * 获得部门业绩目标设置列表
     * @param ids 编号
     * @return 部门业绩目标设置列表
     */
    List<DeptTargetDO> getDeptTargetList(Collection<Long> ids);

    /**
     * 获得部门业绩目标设置分页
     * @param page 分页查询
     * @param query 查询
     * @return 部门业绩目标设置分页
     */
    PageResult<DeptTargetDO> getDeptTargetPage(DeptTargetQueryVO query, PageVO page);

    /**
     * 获得部门业绩目标设置列表, 用于 Excel 导出
     * @param query 查询
     * @return 部门业绩目标设置列表
     */
    List<DeptTargetDO> getDeptTargetList(DeptTargetQueryVO query);

    Map<String, Object> getCreateInitData();

    PageResult<DeptTargetBackVO> getDeptTargetStatisticsPage(DeptTargetQueryVO query, PageVO page);

    PageResult<PersonalDeptTargetDO> getPersonalDeptTargetPage(DeptTargetQueryVO query, PageVO page);

    MyAchievementRetDto myAchievementByPage(DeptTargetQueryVO query, PageVO page);

    MyAchievementRetDto myDeptAchievementByPage(DeptTargetQueryVO query, PageVO pageVO);

    List<MyAchievementDto> getExportDetailExcel(DeptTargetQueryVO exportReqVO);

    List<AchievementStatisticsDto> getExportStatisticsExcel(DeptTargetQueryVO exportReqVO);
}
