package cn.iocoder.yudao.module.sale.service.deptTarget;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.targetLog.TargetLogService;
import cn.iocoder.yudao.module.order.vo.order.OrderBackVO;
import cn.iocoder.yudao.module.sale.convert.deptTarget.DeptTargetConvert;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.DeptTargetDO;
import cn.iocoder.yudao.module.sale.dal.dataobject.deptTarget.PersonalDeptTargetDO;
import cn.iocoder.yudao.module.sale.dal.mysql.deptTarget.DeptTargetMapper;
import cn.iocoder.yudao.module.sale.dto.AchievementStatisticsDto;
import cn.iocoder.yudao.module.sale.dto.MyAchievementDto;
import cn.iocoder.yudao.module.sale.dto.MyAchievementRetDto;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetBackVO;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetQueryVO;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetUpdateReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.PostDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.dept.DeptService;
import cn.iocoder.yudao.module.system.service.dept.PostService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.sale.enums.ErrorCodeConstants.DEPT_TARGET_NOT_EXISTS;

/**
 * 部门业绩目标设置 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class DeptTargetServiceImpl extends AbstractService<DeptTargetMapper, DeptTargetDO> implements DeptTargetService {

    @Resource
    private DeptTargetMapper deptTargetMapper;

    @Resource
    private AdminUserService userService;

    @Resource
    private TargetLogService targetLogService;

    @Resource
    private OrderQueryService orderQueryService;

    @Resource
    private PostService postService;

    @Resource
    private DeptService deptService;

    @Override
    public Long createDeptTarget(DeptTargetCreateReqVO createReqVO) {
        // 插入
        DeptTargetDO deptTarget = DeptTargetConvert.INSTANCE.convert(createReqVO);
        deptTargetMapper.insert(deptTarget);
        // 返回
        return deptTarget.getId();
    }

    @Override
    public void updateDeptTarget(DeptTargetUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateDeptTargetExists(updateReqVO.getId());
        // 更新
        DeptTargetDO updateObj = DeptTargetConvert.INSTANCE.convert(updateReqVO);
        deptTargetMapper.updateById(updateObj);
    }

    @Override
    public void deleteDeptTarget(Long id) {
        // 校验存在
        this.validateDeptTargetExists(id);
        // 删除
        deptTargetMapper.deleteById(id);
    }

    private void validateDeptTargetExists(Long id) {
        if (deptTargetMapper.selectById(id) == null) {
            throw exception(DEPT_TARGET_NOT_EXISTS);
        }
    }

    @Override
    public DeptTargetDO getDeptTarget(Long id) {
        return deptTargetMapper.selectById(id);
    }

    @Override
    public List<DeptTargetDO> getDeptTargetList(Collection<Long> ids) {
        return deptTargetMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<DeptTargetDO> getDeptTargetPage(DeptTargetQueryVO query, PageVO page) {
        return deptTargetMapper.selectPage(page, query);
    }

    @Override
    public List<DeptTargetDO> getDeptTargetList(DeptTargetQueryVO query) {
        return deptTargetMapper.selectList(query);
    }

    @Override
    public Map<String, Object> getCreateInitData() {
        Map<String,Object> map = new HashMap<>();
        map.put("month",getMonthData(LocalDate.now(),24));
        map.put("quarter",getQuarterData(LocalDate.now(),8));
        map.put("year",getYearData(LocalDate.now(),3));
        return map;
    }

    @Override
    public PageResult<DeptTargetBackVO> getDeptTargetStatisticsPage(DeptTargetQueryVO query, PageVO page) {
        page.setField("t1.id");
        IPage<DeptTargetBackVO> mpPage = MyBatisUtils.buildPage(page);
        if ("my".equals(query.getFlag())) {
            LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
            List<DeptDO> depts = deptService.getDeptsByParentIdFromCache(loginUser.getDeptId(), true);
            List<Long> depIds = depts.stream().map(DeptDO::getId).collect(Collectors.toList());
            depIds.add(loginUser.getDeptId());
            query.setDeptIds(depIds);
        }
        deptTargetMapper.getDeptTargetStatisticsPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<PersonalDeptTargetDO> getPersonalDeptTargetPage(DeptTargetQueryVO query, PageVO page) {
        page.setField("t1.id");
        IPage<PersonalDeptTargetDO> mpPage = MyBatisUtils.buildPage(page);
        deptTargetMapper.getPersonalDeptTargetPage(query,mpPage);
        return PageResult.of(mpPage);
    }

    @Override
    public MyAchievementRetDto myAchievementByPage(DeptTargetQueryVO query,
                                                   PageVO page) {

        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        MyAchievementRetDto myAchievementRetDto = new MyAchievementRetDto();
//        List<Long> collect = new ArrayList<>();
//        UserExportReqVO userExportReqVO = new UserExportReqVO();
//        userExportReqVO.setCustomerServiceId(loginUser.getId());
//        List<AdminUserDO> users = userService.getUsers(userExportReqVO);
//        if(users!=null && users.size()>0){
//            collect = users.stream().map(item -> item.getId()).collect(Collectors.toList());
//        }
        page.setField("c.create_time");
        IPage<MyAchievementDto> mpPage = MyBatisUtils.buildPage(page);
        if (query.getId() == null || query.getId() <= 0L) {
            query.setId(loginUser.getId());
        }
        if (StringUtils.isNotBlank(query.getDestWarehouseIds())) {
            List<Integer> destWarehouseIds =
                    Arrays.stream(query.getDestWarehouseIds().split(",")).map(Integer::parseInt).collect(Collectors.toList());
            query.setDestWarehouseIdList(destWarehouseIds);
        }
        List<PostDO> posts =
                postService.getPosts(loginUser.getPostIds());
        List<PostDO> customer_service =
                posts.stream().filter(item -> item.getCode().contains("customer service")&&!item.getCode().contains("documentary")).collect(Collectors.toList());
        if(!"my".equals(query.getFlag()) || (loginUser.getPostIds() != null
                && customer_service!= null &&
                customer_service.size()>0)){

            deptTargetMapper.getMyAchievementByPage(query,mpPage);

            myAchievementRetDto.setMyAchievementDtos(PageResult.of(mpPage));
//            PageVO pageVO = new PageVO();
//            pageVO.setField("c.create_time");
//            pageVO.setRows(Integer.MAX_VALUE);
//            IPage<MyAchievementDto> mpPage1 = MyBatisUtils.buildPage(pageVO);
            List<MyAchievementDto> myAchievementByPage =
                    deptTargetMapper.getMyAchievement(query);
//            IPage<MyAchievementDto> myAchievementByPage = deptTargetMapper.getMyAchievementByPage(query, mpPage1);
            //总数
            myAchievementRetDto.setTotalNum(myAchievementByPage.stream().mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setTotalVolume(myAchievementByPage.stream().map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setTotalWeight(myAchievementByPage.stream().map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //开发用户
            myAchievementRetDto.setDevTotalNum(myAchievementByPage.stream().filter(item -> item.getCustomerType()==1).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setDevTotalVolume(myAchievementByPage.stream().filter(item -> item.getCustomerType()==1).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setDevTotalWeight(myAchievementByPage.stream().filter(item -> item.getCustomerType()==1).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //公司客户
            myAchievementRetDto.setCompanyTotalNum(myAchievementByPage.stream().filter(item -> item.getCustomerType()==2).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setCompanyTotalVolume(myAchievementByPage.stream().filter(item -> item.getCustomerType()==2).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setCompanyTotalWeight(myAchievementByPage.stream().filter(item -> item.getCustomerType()==2).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //新客户
            myAchievementRetDto.setNewTotalNum(myAchievementByPage.stream().filter(item -> "新客户".equals(item.getAchieveType())).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setNewTotalVolume(myAchievementByPage.stream().filter(item -> "新客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setNewTotalWeight(myAchievementByPage.stream().filter(item -> "新客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //旧客户
            myAchievementRetDto.setOldTotalNum(myAchievementByPage.stream().filter(item -> "旧客户".equals(item.getAchieveType())).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setOldTotalVolume(myAchievementByPage.stream().filter(item -> "旧客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setOldTotalWeight(myAchievementByPage.stream().filter(item -> "旧客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
        } else {
            myAchievementRetDto.setMyAchievementDtos(PageResult.of(mpPage));
            myAchievementRetDto.setTotalNum(0);
            myAchievementRetDto.setTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setTotalWeight(BigDecimal.ZERO);
            //开发用户
            myAchievementRetDto.setDevTotalNum(0);
            myAchievementRetDto.setDevTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setDevTotalWeight(BigDecimal.ZERO);
            //公司客户
            myAchievementRetDto.setCompanyTotalNum(0);
            myAchievementRetDto.setCompanyTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setCompanyTotalWeight(BigDecimal.ZERO);
            //新客户
            myAchievementRetDto.setNewTotalNum(0);
            myAchievementRetDto.setNewTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setNewTotalWeight(BigDecimal.ZERO);
            //旧客户
            myAchievementRetDto.setOldTotalNum(0);
            myAchievementRetDto.setOldTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setOldTotalWeight(BigDecimal.ZERO);
        }
//        if(collect.size()>0){

//            List<TargetLogDO> targetLogListByUserIds = targetLogService.getTargetLogListByUserIds(collect,query.getBeginStartTime(),query.getEndStartTime());
//            if(targetLogListByUserIds!=null&&targetLogListByUserIds.size()>0){
//                List<Long> orderIds = targetLogListByUserIds.stream().map(item -> item.getOrderId()).collect(Collectors.toList());
//                OrderQueryVO orderQueryVO = new OrderQueryVO();
//                if(query.getTransportType()!=null&&query.getTransportType()>0L){
//                    orderQueryVO.setTransportId(query.getTransportType().intValue());
//                }
//                if(query.getShippingChannel()!=null&&query.getShippingChannel()>0L){
//                    orderQueryVO.setChannelId(query.getShippingChannel());
//                }
//                orderListByIds = orderService.getOrderListByIds(orderIds, page,orderQueryVO);
//            }
//        }
        return myAchievementRetDto;
    }

    @Override
    public MyAchievementRetDto myDeptAchievementByPage(DeptTargetQueryVO query, PageVO page) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        MyAchievementRetDto myAchievementRetDto = new MyAchievementRetDto();
        page.setField("c.create_time");
        IPage<MyAchievementDto> mpPage = MyBatisUtils.buildPage(page);
        if (query.getId() == null || query.getId() <= 0L) {
            query.setId(loginUser.getId());
        }
        if (StringUtils.isNotBlank(query.getDestWarehouseIds())) {
            List<Integer> destWarehouseIds = Arrays.stream(query.getDestWarehouseIds().split(",")).map(Integer::parseInt).collect(Collectors.toList());
            query.setDestWarehouseIdList(destWarehouseIds);
        }
        List<PostDO> posts = postService.getPosts(loginUser.getPostIds());
        List<PostDO> customer_service = posts.stream().filter(item -> item.getCode().contains("customer service")&&!item.getCode().contains("documentary")).collect(Collectors.toList());
        if(loginUser.getPostIds() != null && customer_service!= null && customer_service.size()>0){
            List<DeptDO> depts = deptService.getDeptsByParentIdFromCache(loginUser.getDeptId(), true);
            List<Long> depIds = depts.stream().map(DeptDO::getId).collect(Collectors.toList());
            depIds.add(loginUser.getDeptId());
            List<AdminUserDO> users = userService.getUsersByDeptIds(depIds);
            List<Long> userIds = new ArrayList<>();
            users.stream().forEach(item -> {
                List<PostDO> postDo = postService.getPosts(item.getPostIds());
                List<PostDO> customerServicePostList = postDo.stream().filter(i -> i.getCode().contains("customer service")&&!i.getCode().contains("documentary")).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(customerServicePostList)) {
                    userIds.add(item.getId());
                }
            });
            query.setUserIds(userIds);
            deptTargetMapper.getMyDeptAchievementByPage(query,mpPage);
            myAchievementRetDto.setMyAchievementDtos(PageResult.of(mpPage));
            List<MyAchievementDto> myAchievementByPage = deptTargetMapper.getMyDeptAchievement(query);
            //总数
            myAchievementRetDto.setTotalNum(myAchievementByPage.stream().mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setTotalVolume(myAchievementByPage.stream().map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setTotalWeight(myAchievementByPage.stream().map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //开发用户
            myAchievementRetDto.setDevTotalNum(myAchievementByPage.stream().filter(item -> item.getCustomerType()==1).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setDevTotalVolume(myAchievementByPage.stream().filter(item -> item.getCustomerType()==1).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setDevTotalWeight(myAchievementByPage.stream().filter(item -> item.getCustomerType()==1).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //公司客户
            myAchievementRetDto.setCompanyTotalNum(myAchievementByPage.stream().filter(item -> item.getCustomerType()==2).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setCompanyTotalVolume(myAchievementByPage.stream().filter(item -> item.getCustomerType()==2).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setCompanyTotalWeight(myAchievementByPage.stream().filter(item -> item.getCustomerType()==2).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //新客户
            myAchievementRetDto.setNewTotalNum(myAchievementByPage.stream().filter(item -> "新客户".equals(item.getAchieveType())).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setNewTotalVolume(myAchievementByPage.stream().filter(item -> "新客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setNewTotalWeight(myAchievementByPage.stream().filter(item -> "新客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
            //旧客户
            myAchievementRetDto.setOldTotalNum(myAchievementByPage.stream().filter(item -> "旧客户".equals(item.getAchieveType())).mapToInt(MyAchievementDto::getSumNum).sum());
            myAchievementRetDto.setOldTotalVolume(myAchievementByPage.stream().filter(item -> "旧客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
            myAchievementRetDto.setOldTotalWeight(myAchievementByPage.stream().filter(item -> "旧客户".equals(item.getAchieveType())).map(MyAchievementDto::getCompleteWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
        } else {
            myAchievementRetDto.setMyAchievementDtos(PageResult.of(mpPage));
            myAchievementRetDto.setTotalNum(0);
            myAchievementRetDto.setTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setTotalWeight(BigDecimal.ZERO);
            //开发用户
            myAchievementRetDto.setDevTotalNum(0);
            myAchievementRetDto.setDevTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setDevTotalWeight(BigDecimal.ZERO);
            //公司客户
            myAchievementRetDto.setCompanyTotalNum(0);
            myAchievementRetDto.setCompanyTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setCompanyTotalWeight(BigDecimal.ZERO);
            //新客户
            myAchievementRetDto.setNewTotalNum(0);
            myAchievementRetDto.setNewTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setNewTotalWeight(BigDecimal.ZERO);
            //旧客户
            myAchievementRetDto.setOldTotalNum(0);
            myAchievementRetDto.setOldTotalVolume(BigDecimal.ZERO);
            myAchievementRetDto.setOldTotalWeight(BigDecimal.ZERO);
        }
        return myAchievementRetDto;
    }

    @Override
    public List<MyAchievementDto> getExportDetailExcel(DeptTargetQueryVO exportReqVO) {
        List<MyAchievementDto> myAchievement = deptTargetMapper.getMyAchievement(exportReqVO);
        myAchievement.stream().forEach(item -> {
            item.setOrderStatus(item.getStatusMsg());
        });
        return myAchievement;
    }

    @Override
    public List<AchievementStatisticsDto> getExportStatisticsExcel(DeptTargetQueryVO exportReqVO) {
        return deptTargetMapper.getExportStatisticsExcel(exportReqVO);
    }

    public static List<Map> getMonthData(LocalDate localDate,int num){
        List<Map> list = new ArrayList();
        for(int i=0;i<num;i++){
            Map map = new HashMap();
            LocalDate date = localDate.plusMonths(i);
            String str = date.toString();
            LocalDate startDate = date.with(TemporalAdjusters.firstDayOfMonth());
            LocalDate endDate = date.with(TemporalAdjusters.lastDayOfMonth());
            map.put("name",str.substring(0, 4)+"年"+str.substring(5,7)+"月");
            map.put("startDate",startDate.toString());
            map.put("endDate",endDate.toString());
            list.add(map);
        }
        return list;
    }

    public static List<Map> getQuarterData(LocalDate localDate,int num){
        List<Map> list = new ArrayList();
        for(int i=0;i<num;i++){
            Map map = new HashMap();
            LocalDate date = localDate.plusMonths(3*i);
            String str = date.toString();
            String year = str.substring(0,4);
            String month = str.substring(5,7);
            String startDate = "";
            String endDate = "";
            int quarter = (Integer.valueOf(month)+2)/3;
            if(quarter==1){
                startDate = year+"-01-01";
                endDate = year+"-03-31";
            }else if(quarter==2){
                startDate = year+"-04-01";
                endDate = year+"-06-30";
            }else if(quarter==3){
                startDate = year+"-07-01";
                endDate = year+"-09-30";
            }else{
                startDate = year+"-10-01";
                endDate = year+"-12-31";
            }
            map.put("name",year+"年"+quarter+"季度");
            map.put("startDate",startDate);
            map.put("endDate",endDate);
            list.add(map);
        }
        return list;
    }

    public static List<Map> getYearData(LocalDate localDate,int num){
        List<Map> list = new ArrayList();
        for(int i=0;i<num;i++){
            Map map = new HashMap();
            LocalDate date = localDate.plusYears(i);
            String str = date.toString();
            LocalDate startDate = date.with(TemporalAdjusters.firstDayOfYear());
            LocalDate endDate = date.with(TemporalAdjusters.lastDayOfYear());
            String year = str.substring(0,4);
            map.put("name",year+"年");
            map.put("startDate",startDate.toString());
            map.put("endDate",endDate.toString());
            list.add(map);
        }
        return list;
    }
}
