package cn.iocoder.yudao.module.sale.service.offer;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.sale.vo.offer.prod.OfferProdCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.prod.OfferProdExportReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.prod.OfferProdPageReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.prod.OfferProdUpdateReqVO;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferProdDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 报价商品详情 Service 接口
 *
 * @author 系统管理员
 */
public interface OfferProdService extends IService<OfferProdDO> {

    /**
     * 创建报价商品详情
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOfferProd(@Valid OfferProdCreateReqVO createReqVO);

    /**
     * 更新报价商品详情
     *
     * @param updateReqVO 更新信息
     */
    void updateOfferProd(@Valid OfferProdUpdateReqVO updateReqVO);

    /**
     * 删除报价商品详情
     *
     * @param id 编号
     */
    void deleteOfferProd(Long id);

    /**
     * 获得报价商品详情
     *
     * @param id 编号
     * @return 报价商品详情
     */
    OfferProdDO getOfferProd(Long id);

    /**
     * 获得报价商品详情列表
     *
     * @param ids 编号
     * @return 报价商品详情列表
     */
    List<OfferProdDO> getOfferProdList(Collection<Long> ids);

    /**
     * 获得报价商品详情分页
     *
     * @param pageReqVO 分页查询
     * @return 报价商品详情分页
     */
    PageResult<OfferProdDO> getOfferProdPage(OfferProdPageReqVO pageReqVO);

    /**
     * 获得报价商品详情列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 报价商品详情列表
     */
    List<OfferProdDO> getOfferProdList(OfferProdExportReqVO exportReqVO);

}
