package cn.iocoder.yudao.module.sale.service.offer;

import java.io.IOException;
import java.util.*;
import javax.servlet.http.HttpServletResponse;
import javax.validation.*;

import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionDO;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.sale.vo.offer.*;
import cn.iocoder.yudao.module.sale.vo.offer.prod.OfferProdCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.offerApproval.OfferApprovalBackInfoVO;

/**
 * 报价单管理 Service 接口
 *
 * @author 系统管理员
 */
public interface OfferService extends IService<OfferDO> {

    /**
     * 创建报价单管理
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOffer(@Valid OfferCreateReqVO createReqVO);

    /**
     * 更新报价单管理
     *
     * @param updateReqVO 更新信息
     */
    void updateOffer(@Valid OfferUpdateReqVO updateReqVO);


    /**
     * 删除报价单管理
     *
     * @param id 编号
     */
    void deleteOffer(Long id);

    /**
     * 获得报价单管理
     *
     * @param id 编号
     * @return 报价单管理
     */
    OfferDO getOffer(Long id);

    /**
     * 获得报价单管理列表
     *
     * @param ids 编号
     * @return 报价单管理列表
     */
    List<OfferDO> getOfferList(Collection<Long> ids);

    /**
     * 获得报价单管理分页
     *
     * @param pageReqVO 分页查询
     * @return 报价单管理分页
     */
    PageResult<OfferDO> getOfferPage(OfferPageReqVO pageReqVO);

    /**
     * 获得报价单管理列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 报价单管理列表
     */
    List<OfferDO> getOfferList(OfferExportReqVO exportReqVO);


    OfferRespVO info(Long offerId);

    /**
     * 特价申请管理
     *
     * @param specialApplyVO 特价申请信息
     */
    void specialApplyOffer(OfferSpecialApplyVO specialApplyVO);

    /**
     * 特价申请业务表单详情
     *
     * @param approveId 业务ID
     * @return 特价申请详情
     */
    OfferApprovalBackInfoVO approvalOfferInfo(Long approveId);

    /**
     * 特价申请业务表单详情
     *
     * @param type        类型：1 特价优惠 2 佣金申请
     * @param offerProdId 报价单商品ID
     * @param isResubmit  是否重新提交获取详情
     * @return 特价申请详情
     */
    OfferApprovalBackInfoVO approvalOfferSpecialByOfferProdId(Integer type, Long offerProdId, boolean isResubmit);


    /**
     * @param approveId 表单业务ID
     * @param result    1 处理中  2 通过 3 不通过 4 已取消
     */
    void approvalOffer(String approveId, Integer result);

    /**
     * 跟新报价单结果
     *
     * @param offerResultVO 跟新报价单结果
     */
    Long updateOfferResult(OfferResultVO offerResultVO);

    /**
     * 获得报价单管理分页
     *
     * @param pageReqVO 分页查询
     * @return 报价单管理分页
     */
    PageResult<OfferRespVO> offerPage(OfferPageReqVO pageReqVO);

//    /**
//     * 获得报价单管理分页
//     *
//     * @param pageReqVO 分页查询
//     * @return 报价单管理分页
//     */
//    PageResult<OfferRespVO> deptOfferPage(OfferPageReqVO pageReqVO);

    void cancelOfferSpecialApproval(Long offerApprovalId, String reason);

    void cancelOffer(Long id);

    void recoveryOffer(Long id);

    void exportOfferExcel(HttpServletResponse response, OfferPageReqVO pageVO) throws IOException;

    /**
     * 检查订单是否有关联的报价单
     *
     * @param orderId 订单ID
     * @return 有则返回报价单详情，无则返回null
     */
    OfferRespVO getOfferByOrderId(Long orderId);

    /**
     * 当订单已关联报价单时会给错误提示
     *
     * @param orderId 订单ID
     * @return 有则返回可关联报价单列表，无则返回null
     */
    List<OfferRespVO> getOfferListByOrderId(Long orderId);


    /**
     * 当订单已关联报价单时会给错误提示
     *
     * @param queryVO 查询参数
     * @return 有则返回可关联报价单列表，无则返回null
     */
    List<OfferRespVO> getOfferSelect(OrderAssociationOfferQueryVO queryVO);

    /**
     * 订单关联报价单
     *
     * @param offerId 报价单id
     * @param orderId 订单ID
     */
    void associationOfferAndOrder(Long offerId, Long orderId);

    EstCostVO getOfferCostByOrderId(Long orderId);


    /**
     * 报价单操作日志添加
     *
     * @param offerId        报价单ID
     * @param number         报价单号
     * @param customerId     客户ID
     * @param followUpMethod 方式 1：新建报价 2 修改报价 3特价申请 4电话 5网络 6报价结束
     * @param userId         操作用户ID(业务员ID)
     * @param result         处理结果说明
     */
    void addOfferLog(Long offerId, String number, Long customerId, Integer followUpMethod, String userId, String result);

    /**
     * 跟进客户查询其关联的报价单
     *
     * @param customerId 客户id
     * @param searchKey  关键字搜索
     * @return
     */
    List<OfferRespVO> getCustomerOfferSelect(Long customerId, String searchKey);
}
