package cn.iocoder.yudao.module.sale.service.offer;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportExportReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportPageReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportUpdateReqVO;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferTransportDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;

/**
 * 报价单运输方式 Service 接口
 *
 * @author 系统管理员
 */
public interface OfferTransportService extends IService<OfferTransportDO> {

    /**
     * 创建报价单运输方式
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOfferTransport(@Valid OfferTransportCreateReqVO createReqVO);

    /**
     * 更新报价单运输方式
     *
     * @param updateReqVO 更新信息
     */
    void updateOfferTransport(@Valid OfferTransportUpdateReqVO updateReqVO);

    /**
     * 删除报价单运输方式
     *
     * @param id 编号
     */
    void deleteOfferTransport(Long id);

    /**
     * 获得报价单运输方式
     *
     * @param id 编号
     * @return 报价单运输方式
     */
    OfferTransportDO getOfferTransport(Long id);

    /**
     * 获得报价单运输方式列表
     *
     * @param ids 编号
     * @return 报价单运输方式列表
     */
    List<OfferTransportDO> getOfferTransportList(Collection<Long> ids);

    /**
     * 获得报价单运输方式分页
     *
     * @param pageReqVO 分页查询
     * @return 报价单运输方式分页
     */
    PageResult<OfferTransportDO> getOfferTransportPage(OfferTransportPageReqVO pageReqVO);

    /**
     * 获得报价单运输方式列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 报价单运输方式列表
     */
    List<OfferTransportDO> getOfferTransportList(OfferTransportExportReqVO exportReqVO);

}
