package cn.iocoder.yudao.module.sale.service.offer.impl;

import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.follow.CustomerFollowDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customer.complaint.follow.CustomerFollowService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowBaseVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowPageReqVO;
import cn.iocoder.yudao.module.order.vo.order.OrderBackPageVO;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.module.sale.dal.mysql.offer.OfferMapper;
import cn.iocoder.yudao.module.sale.service.offer.OfferLogService;
import cn.iocoder.yudao.module.sale.vo.offer.log.OfferLogBackVO;
import cn.iocoder.yudao.module.sale.vo.offer.log.OfferLogCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.log.OfferLogQueryVO;
import cn.iocoder.yudao.module.sale.vo.offer.log.OfferLogUpdateReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.sale.convert.offer.OfferLogConvert;
import cn.iocoder.yudao.module.sale.dal.mysql.offer.OfferLogMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.sale.enums.ErrorCodeConstants.*;

/**
 * 报价单跟进日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class OfferLogServiceImpl extends AbstractService<OfferLogMapper, OfferLogDO> implements OfferLogService {

    @Autowired
    private OfferLogMapper offerLogMapper;
    @Autowired
    private OfferMapper offerMapper;
    @Autowired
    private CustomerContactsService customerContactsService;
    @Autowired
    private AdminUserService adminUserService;

    @Autowired
    private CustomerFollowService customerFollowService;

    @Override
    public Long createOfferLog(OfferLogCreateReqVO createReqVO) {
        Long sysUserId = SecurityFrameworkUtils.getLoginUserId();
        CustomerContactsDO contactsDO = customerContactsService.getById(createReqVO.getContacts());
        if (contactsDO == null) {
            throw exception(CUSTOMER_CONTACTS_NOT_EXISTS);
        }
        // 插入
        CustomerFollowCreateReqVO customerFollowCreateReqVO = new CustomerFollowCreateReqVO();

//        OfferLogDO offerLog = OfferLogConvert.INSTANCE.convert(createReqVO);
        // 校验报价单
        OfferDO offerDO = offerMapper.selectById(createReqVO.getOfferId());
        if (offerDO == null) {
            throw exception(OFFER_NOT_EXISTS);
        }
        if (offerDO.getStatus() != 3 && offerDO.getStatus() != 7) {
            throw exception(OFFER_FOLLOW_UP_STATUS_ERROR);
        }
        customerFollowCreateReqVO.setFollowType(4);
        customerFollowCreateReqVO.setFollowTime(createReqVO.getFollowUpTime());
        customerFollowCreateReqVO.setCustomerId(offerDO.getRelationId());
        customerFollowCreateReqVO.setBizNo(offerDO.getNumber());
        customerFollowCreateReqVO.setBizId(offerDO.getOfferId());
        customerFollowCreateReqVO.setContactName(contactsDO.getName());
        customerFollowCreateReqVO.setFollowMethod(createReqVO.getFollowUpMethod());
        customerFollowCreateReqVO.setFeedback(createReqVO.getCustomerFeedback());
        customerFollowCreateReqVO.setResult(createReqVO.getProcessingResults());
        if (StringUtils.isBlank(createReqVO.getFollowUpSalesmanId())) {
            customerFollowCreateReqVO.setFollowUserId(sysUserId);
        } else {
            customerFollowCreateReqVO.setFollowUserId(Long.valueOf(createReqVO.getFollowUpSalesmanId()));
        }
        Long followId = customerFollowService.createCustomerFollow(customerFollowCreateReqVO);
//        offerLogMapper.insert(offerLog);
        if (offerDO.getStatus() == 3) {
            offerDO.setStatus(7);
            offerMapper.updateById(offerDO);
        }
        // 返回
        return followId;
    }

    @Override
    public void updateOfferLog(OfferLogUpdateReqVO updateReqVO) {
        // 校验存在
//        this.validateOfferLogExists(updateReqVO.getOfferLogId());
        // 更新
        OfferLogDO updateObj = OfferLogConvert.INSTANCE.convert(updateReqVO);
        offerLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteOfferLog(Long id) {
        // 校验存在
//        this.validateOfferLogExists(id);
        // 删除
        offerLogMapper.deleteById(id);
    }

//    private void validateOfferLogExists(Long id) {
//        if (offerLogMapper.selectById(id) == null) {
//            throw exception(OFFER_LOG_NOT_EXISTS);
//        }
//    }

    @Override
    public OfferLogDO getOfferLog(Long id) {
        return offerLogMapper.selectById(id);
    }

    @Override
    public List<OfferLogBackVO> getOfferLogListByOfferId(Long offerId) {
        List<CustomerFollowDO> followDOList = customerFollowService.offerFollowList(offerId);
        if (CollectionUtil.isNotEmpty(followDOList)) {
            return followDOList.stream().map(OfferLogServiceImpl::getOfferLogAsBackVO).collect(Collectors.toList());
        }
        return null;
//        return offerLogMapper.selectList(new LambdaQueryWrapper<OfferLogDO>().eq(OfferLogDO::getOfferId, offerId).orderByDesc(OfferLogDO::getOfferLogId));
    }

    @NotNull
    private static OfferLogBackVO getOfferLogAsBackVO(CustomerFollowDO followDO) {
        OfferLogBackVO backVO = new OfferLogBackVO();
        backVO.setType(2);
        backVO.setFollowUpTime(followDO.getFollowTime());
        backVO.setCustomerId(followDO.getCustomerId());
        backVO.setOfferId(followDO.getBizId());
        backVO.setNumber(followDO.getBizNo());
        backVO.setContactName(followDO.getContactName());
        backVO.setFollowUpMethod(followDO.getFollowMethod());
        backVO.setCustomerFeedback(followDO.getFeedback());
        backVO.setProcessingResults(followDO.getResult());
        backVO.setFollowUpSalesmanId(String.valueOf(followDO.getFollowUserId()));
        backVO.setCreateTime(followDO.getCreateTime());
        backVO.setOfferLogId(followDO.getId());
        backVO.setFollowUpSalesmanName(followDO.getFollowUserName());
        return backVO;
    }

    @Override
    public PageResult<OfferLogBackVO> getOfferLogPage(OfferLogQueryVO query, PageVO page) {
        if (query.getType() == 2) {
            CustomerFollowPageReqVO pageReqVO = new CustomerFollowPageReqVO();
            pageReqVO.setBizId(query.getOfferId());
            pageReqVO.setBizNo(query.getNumber());
            pageReqVO.setFollowType("4");
            pageReqVO.setFollowUserId(StringUtils.isNotBlank(query.getFollowUpSalesmanId())? Long.valueOf(query.getFollowUpSalesmanId()): null);
            pageReqVO.setBeginCreateTime(query.getBeginCreateTime());
            pageReqVO.setEndCreateTime(query.getEndCreateTime());
            pageReqVO.setPageNo(page.getPage());
            pageReqVO.setPageSize(page.getRows());
            PageResult<CustomerFollowDO> pageFollowResult = customerFollowService.customerFollowPage(pageReqVO);

            PageResult<OfferLogBackVO> pageResult = new PageResult<>();
            if (CollectionUtil.isNotEmpty(pageFollowResult.getList())) {
                pageResult.setList(pageFollowResult.getList().stream().map(OfferLogServiceImpl::getOfferLogAsBackVO).collect(Collectors.toList()));
            }else {
                pageResult.setList(new ArrayList<>());
            }
            pageResult.setTotal( pageFollowResult.getTotal() );
            pageResult.setRows( pageFollowResult.getRows() );
            pageResult.setPage( pageFollowResult.getPage() );
            pageResult.setPages( pageFollowResult.getPages() );
            return pageResult;
        } else {
            page.setField("offer_log_id");
            PageResult<OfferLogDO> pageResult = offerLogMapper.selectPage(page, query);
            if (CollectionUtil.isNotEmpty(pageResult.getList())) {
                pageResult.setList(pageResult.getList().stream().peek(log -> {
                    if (StringUtils.isNotBlank(log.getContacts())) {
                        CustomerContactsDO contactsDO = customerContactsService.getById(log.getContacts());
                        if (Objects.nonNull(contactsDO)) {
                            log.setContactName(StringUtils.isBlank(contactsDO.getName()) ? "未知" : contactsDO.getName());
                        }
                    }
                    if (StringUtils.isNotBlank(log.getFollowUpSalesmanId())) {
                        AdminUserDO adminUserDO = adminUserService.getUser(Long.valueOf(log.getFollowUpSalesmanId()));
                        if (Objects.nonNull(adminUserDO)) {
                            log.setFollowUpSalesmanName(StringUtils.isBlank(adminUserDO.getNickname()) ? "未知" : adminUserDO.getNickname());
                        }
                    }
                }).collect(Collectors.toList()));
            }

            return OfferLogConvert.INSTANCE.convertPage(pageResult);
        }
    }

    @Override
    public List<OfferLogDO> getOfferLogList(OfferLogQueryVO query) {
        return offerLogMapper.selectList(query);
    }

    @Override
    public PageResult<OfferLogBackVO> getCustomerOfferLogPage(OfferLogQueryVO query, PageVO page) {
        page.setField("o.offer_log_id");
        query.setLang(I18nMessage.getLang());
        IPage<OfferLogBackVO> mpPage = MyBatisUtils.buildPage(page);
        offerLogMapper.getCustomerOfferLogPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public OfferLogBackVO getCustomerFollow(Long id) {
        CustomerFollowDO followDO = customerFollowService.getById(id);
        return getOfferLogAsBackVO(followDO);
    }
}
