package cn.iocoder.yudao.module.sale.service.offer.impl;

import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportCreateReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportExportReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportPageReqVO;
import cn.iocoder.yudao.module.sale.vo.offer.transport.OfferTransportUpdateReqVO;
import cn.iocoder.yudao.module.sale.service.offer.OfferTransportService;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferTransportDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.sale.convert.offer.OfferTransportConvert;
import cn.iocoder.yudao.module.sale.dal.mysql.offer.OfferTransportMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;

/**
 * 报价单运输方式 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class OfferTransportServiceImpl extends AbstractService<OfferTransportMapper, OfferTransportDO> implements OfferTransportService {

    @Resource
    private OfferTransportMapper offerTransportMapper;

    @Override
    public Long createOfferTransport(OfferTransportCreateReqVO createReqVO) {
        // 插入
        OfferTransportDO offerTransport = OfferTransportConvert.INSTANCE.convert(createReqVO);
        offerTransportMapper.insert(offerTransport);
        // 返回
        return offerTransport.getOfferTransportId();
    }

    @Override
    public void updateOfferTransport(OfferTransportUpdateReqVO updateReqVO) {
        // 校验存在
//        this.validateOfferTransportExists(updateReqVO.getOfferTransportId());
        // 更新
        OfferTransportDO updateObj = OfferTransportConvert.INSTANCE.convert(updateReqVO);
        offerTransportMapper.updateById(updateObj);
    }

    @Override
    public void deleteOfferTransport(Long id) {
        // 校验存在
//        this.validateOfferTransportExists(id);
        // 删除
        offerTransportMapper.deleteById(id);
    }

//    private void validateOfferTransportExists(Long id) {
//        if (offerTransportMapper.selectById(id) == null) {
//            throw exception(OFFER_TRANSPORT_NOT_EXISTS);
//        }
//    }

    @Override
    public OfferTransportDO getOfferTransport(Long id) {
        return offerTransportMapper.selectById(id);
    }

    @Override
    public List<OfferTransportDO> getOfferTransportList(Collection<Long> ids) {
        return offerTransportMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<OfferTransportDO> getOfferTransportPage(OfferTransportPageReqVO pageReqVO) {
        return offerTransportMapper.selectPage(pageReqVO);
    }

    @Override
    public List<OfferTransportDO> getOfferTransportList(OfferTransportExportReqVO exportReqVO) {
        return offerTransportMapper.selectList(exportReqVO);
    }

}
