package cn.iocoder.yudao.module.sale.vo.offer;

import cn.iocoder.yudao.framework.common.validation.Marks;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import io.swagger.annotations.*;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.*;


/**
* 报价单管理 Base VO，提供给添加、修改、详细的子 VO 使用
* 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
*/
@Data
public class  OfferBaseVO {

    @ApiModelProperty(value = "报价单号", required = true)
    private String number;

    @ApiModelProperty(value = "订单id")
    private Long orderId;

    @ApiModelProperty(value = "订单编号")
    private String orderNo;

    @ApiModelProperty(value = "发货人id", required = true)
//    @NotNull(message = "客户联系人id，发货人不能为空")
    private Long consignorId;

    @ApiModelProperty(value = "收货人id", required = true)
//    @NotNull(message = "客户联系人id,收货人不能为空")
    private Long consigneeId;

    @ApiModelProperty(value = "唛头")
    @NotBlank(message = "marks.not.null")
    @Length(max = 20, message = "marks.length.max")
//    @Marks(message = "marks.format.incorrect")
    private String marks;

    @ApiModelProperty(value = "订单类型(可多选)：1 集运服务 2 海外仓")
    private String serviceType;

    @ApiModelProperty(value = "报关类别：我司全代：1，自单代报：2，混合报关：3")
    private Integer customsType;

    @ApiModelProperty(value = "始发地")
    @NotNull(message = "departure.not.exists")
    private Long departureId;

//    @ApiModelProperty(value = "始发地信息")
//    private String departure;

    @ApiModelProperty(value = "目的地")
    @NotNull(message = "objective.not.exists")
    private Long objectiveId;

    @ApiModelProperty(value = "目的地国ID")
    @NotNull(message = "目的国不能为空")
    private Long destCountryId;

//    @ApiModelProperty(value = "目的地信息")
//    private String objective;

    @ApiModelProperty(value = "预计结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date stopTime;

    @ApiModelProperty(value = "控货")
    private Boolean control;

    @ApiModelProperty(value = "竞争对手")
    private String competitor;

    @ApiModelProperty(value = "重要程度")
    private Integer importance;

    @ApiModelProperty(value = "价格有效期开始")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date startTime;

    @ApiModelProperty(value = "价格有效期结束")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date endTime;

    @ApiModelProperty(value = "交货方式：FOB(离岸价)，CIF(到岸价)，CNF(成本加运费)，EXW(出厂价)")
    private Integer tradeType;

    @ApiModelProperty(value = "交货港口、地点")
    private String tradeAdress;

    @ApiModelProperty(value = "备注")
    private String remarks;

    @ApiModelProperty(value = "状态：取消报价：0，特价审批中：1，草稿：2 需求确认：3，赢单：4，输单：5，报价完成：6，跟进中: 7")
    private Integer status;

    @ApiModelProperty(value = "取消报价前状态")
    private Integer oldStatus;

    @ApiModelProperty(value = "报价关联客户，发货人：1，收货人：2")
    private Integer relation;

    @ApiModelProperty(value = "报价关联客户id")
    private Long relationId;

    @ApiModelProperty(value = "客户经理ID")
    private Long salesmanId;

    @ApiModelProperty(value = "客户编号")
    private String customerNumber;

    @ApiModelProperty(value = "客户经理所属部门ID")
    private Long deptId;

    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ApiModelProperty(value = "入仓类型类型: 1 一次入仓 2 分批入仓")
    private Integer warehousingType;

    @ApiModelProperty(value = "原因")
    private String reason;

    @ApiModelProperty(value = "其他费用")
    private BigDecimal otherFee;

    @ApiModelProperty(value = "其他费用货币ID")
    private Integer otherFeeCurrencyId;
    @ApiModelProperty(value = "预计费用清单")
    private String estCost;

    @ApiModelProperty(value = "预计费用清单")
    private EstCostVO estCostVO;

    @ApiModelProperty(value = "站内信状态，0未发送，1已发送", required = true)
    @NotNull(message = "站内信状态，0未发送，1已发送不能为空")
    private Boolean sendstatus;

    @ApiModelProperty(value = "运输id")
    @NotNull(message = "线路id不能为空")
    private Integer transportId;

    @ApiModelProperty(value = "线路id", required = true)
    @NotNull(message = "线路id不能为空")
    private Long lineId;

    @ApiModelProperty(value = "渠道id")
    private Long channelId;

    @ApiModelProperty(value = "跟进业务员")
    private String followUpSalesmanId;

    public void setEstCostVO() {
        this.estCostVO = JSONObject.parseObject(this.estCost, EstCostVO.class);
    }

    public EstCostVO getEstCostVO() {
        return JSONObject.parseObject(this.estCost, EstCostVO.class);
    }

    @ApiModelProperty(value = "商户洽谈的跟进记录数量")
    private Integer logCount;


}
