package cn.iocoder.yudao.module.sale.vo.offer;

import cn.iocoder.yudao.framework.excel.annotations.DictFormat;
import cn.iocoder.yudao.framework.excel.convert.DictConvert;
import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

import static cn.iocoder.yudao.module.bpm.enums.DictTypeConstants.TRANSPORT_TYPE;

/**
 * 报价单管理 Excel VO
 *
 * @author 系统管理员
 */
@Data
public class OfferExcelVO {

    @ExcelIgnore
    private Long offerId;

    @ExcelProperty(value = "报价单号", index = 0)
    private String number;

    @ExcelIgnore
    private Long orderId;

    @ExcelProperty(value = "订单编号",index = 1)
    private String orderNo;

    @ExcelIgnore
    private Long consignorId;

    @ExcelIgnore
    private Long consigneeId;

    @ExcelIgnore
    @ApiModelProperty("唛头")
    private String marks;

    @ExcelIgnore
    @ApiModelProperty(value = "订单类型(可多选)：1 集运服务 2 海外仓")
    private String serviceType;

    @ExcelIgnore
    @ApiModelProperty("报关类别：我司全代：1，自单代报：2，混合报关：3")
    private Integer customsType;

    @ExcelIgnore
    @ApiModelProperty("始发地")
    private Long departureId;

    @ExcelIgnore
    @ApiModelProperty("始发地信息")
    private String departure;

    @ExcelIgnore
    private Long objectiveId;

    @ExcelIgnore
    @ApiModelProperty("目的地信息")
    private String objective;

    @ExcelProperty(value = "预计结束时间", index = 6)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date stopTime;

    @ExcelIgnore
    @ApiModelProperty("控货")
    private Boolean control;

    @ExcelIgnore
    @ApiModelProperty("竞争对手")
    private String competitor;

    @ExcelIgnore
    @ApiModelProperty("重要程度")
    private Integer importance;

    @ExcelIgnore
    @ApiModelProperty("价格有效期开始")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date startTime;

    @ExcelIgnore
    @ApiModelProperty("价格有效期结束")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date endTime;

    @ExcelIgnore
    @ApiModelProperty("交货方式：FOB(离岸价)，CIF(到岸价)，CNF(成本加运费)，EXW(出厂价)")
    private Integer tradeType;

    @ExcelIgnore
    @ApiModelProperty("交货港口、地点")
    private String tradeAdress;

    @ExcelIgnore
    @ApiModelProperty("备注")
    private String remarks;

    @ExcelProperty(value = "销售阶段", index = 5, converter = DictConvert.class)
    @DictFormat("offer_status") // TODO 代码优化：建议设置到对应的 XXXDictTypeConstants 枚举类中
    @ApiModelProperty("状态：取消报价：0，特价审批中：1，草稿：2 需求确认：3，赢单：4，输单：5，报价完成：6，跟进中: 7")
    private Integer status;

    @ExcelIgnore
    private Integer oldStatus;

    @ExcelIgnore
    @ApiModelProperty("报价关联客户，发货人：1，收货人：2")
    private Integer relation;

    @ExcelIgnore
    @ApiModelProperty("报价关联客户id")
    private Long relationId;

    @ExcelIgnore
    @ApiModelProperty(value = "客户经理ID")
    private Long salesmanId;

    @ExcelIgnore
    @ApiModelProperty(value = "客户经理所属部门ID")
    private Long deptId;

    @ExcelIgnore
    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ExcelIgnore
    @ApiModelProperty(value = "入仓类型类型")
    private Integer warehousingType;

//    @ApiModelProperty(value = "原因")
//    private String reason;


    @ExcelIgnore
    private Boolean sendstatus;

//    @ExcelProperty("创建时间")
//    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
//    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
//    private Date createTime;

    @ExcelProperty(value = "运输方式", index = 7, converter = DictConvert.class)
    @DictFormat(TRANSPORT_TYPE)
    @ApiModelProperty(value = "运输id")
    private Integer transportId;


    @ApiModelProperty(value = "线路id", required = true)
    @ExcelIgnore
    private Long lineId;

    @ExcelIgnore
    @ApiModelProperty(value = "渠道id")
    private Long channelId;

    @ExcelIgnore
    @ApiModelProperty("跟进业务员")
    private String followUpSalesmanId;

    @ExcelProperty(value = "始发地",index = 3)
    @ApiModelProperty(value = "始发地")
    private String departureName;

    @ExcelProperty(value = "目的地", index = 4)
    @ApiModelProperty(value = "目的地")
    private String objectiveName;

    @ExcelProperty(value = "客户名称", index = 2)
    @ApiModelProperty(value = "客户名称")
    private String relationName;

    @ExcelProperty(value = "客户经理", index = 9)
    @ApiModelProperty(value = "业务员")
    private String creatorName;


    @ExcelProperty(value = "预计销售额", index = 8)
    @ApiModelProperty(value = "预计费用清单")
    private String estCost;

}
