package cn.iocoder.yudao.module.sale.vo.offer.prod;

import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import org.springframework.format.annotation.DateTimeFormat;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 报价商品详情 Base VO，提供给添加、修改、详细的子 VO 使用
 * 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
 */
@Data
public class OfferProdBaseVO {


    @ApiModelProperty(value = "报价单ID", required = true)
    private Long offerId;

    @ApiModelProperty(value = "商品ID", required = true)
    @NotNull(message = "商品ID不能为空")
    private Long prodId;

    @ApiModelProperty(value = "产品中文标题")
    private String prodTitleZh;

    @ApiModelProperty(value = "产品英文标题")
    private String prodTitleEn;

    @ApiModelProperty(value = "商品第三方链接")
    private String link;

    @ApiModelProperty(value = "运输id", required = true)
    @NotNull(message = "运输id不能为空")
    private Integer transportId;

    @ApiModelProperty(value = "线路id", required = true)
    @NotNull(message = "线路id不能为空")
    private Long lineId;

    @ApiModelProperty(value = "渠道id")
    private Long channelId;

    @ApiModelProperty(value = "产品类型")
    private Integer productType;

    @ApiModelProperty(value = "是否品牌(0 否 1 是)", required = true)
    private Integer brand;

    @ApiModelProperty(value = "品牌(0 无牌 1 有牌 2 中性)")
    private Integer brandType;

    @ApiModelProperty(value = "数量", required = true)
    @NotNull(message = "数量不能为空")
    private Integer quantity;

    @ApiModelProperty(value = "件数", required = true)
    @NotNull(message = "件数不能为空")
    private Integer num;

    @ApiModelProperty(value = "包装单位", required = true)
    @NotNull(message = "包装单位不能为空")
    private String unit;

    @ApiModelProperty(value = "箱规(格式：2*2*2.5 以*连接长*宽*高)")
    private String boxGauge;

    @ApiModelProperty(value = "体积(根据箱规计算)")
    private BigDecimal volume;

    @ApiModelProperty(value = "重量")
    private BigDecimal weight;

    @ApiModelProperty(value = "货值")
    private BigDecimal worth;

    @ApiModelProperty(value = "货值金额单位ID(默认美元)")
    private Integer worthCurrency;

    @ApiModelProperty(value = "商品特性(可选多个特性ids)")
    private String prodAttrIds;

    @ApiModelProperty(value = "货物类型")
    private Integer goodsType;

    @ApiModelProperty(value = "易碎：是、否")
    private Integer fragile;

    @ApiModelProperty(value = "是否特价")
    private Boolean specialPriceType;

    @ApiModelProperty(value = "运费单价(成交)")
    private BigDecimal oneSeaFreight;

//    @ApiModelProperty(value = "销售价运费")
//    private BigDecimal saleSeaFreight;

    @ApiModelProperty(value = "成本价运费")
    private BigDecimal costSeaFreight;

    @ApiModelProperty(value = "运费(小计)")
    private BigDecimal seaFreight;

    @ApiModelProperty(value = "原来运费(原始-原价)")
    private BigDecimal originalSeaFreight;

    @ApiModelProperty(value = "运费用金额单位ID")
    private Integer seaFreightCurrency;

    @ApiModelProperty(value = "运费体积单位")
    private Long seaFreightVolume;

    @ApiModelProperty(value = "清关费单价(成交-销售)")
    private BigDecimal oneClearanceFreight;

    @ApiModelProperty(value = "清关费(小计)")
    private BigDecimal clearanceFreight;

    @ApiModelProperty(value = "原来清关费(原始)")
    private BigDecimal originalClearanceFreight;

    @ApiModelProperty(value = "清关费用金额单位ID")
    private Integer clearanceFreightCurrency;

    @ApiModelProperty(value = "清关费体积单位")
    private Long clearanceFreightVolume;

    @ApiModelProperty(value = "预计销售额")
    private BigDecimal estimatedSales;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ApiModelProperty(value = "计费方式：0 运费/清关费 1 全包价")
    private Integer charging;

    @ApiModelProperty(value = "状态：0 正常 1 审批中 2 审批通过 3 审批未通过")
    private Integer status;


    @ApiModelProperty("快递单号")
    private String expressNo;

    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ApiModelProperty("明佣金额")
    private BigDecimal plainCommission = BigDecimal.ZERO;

    @ApiModelProperty("暗佣金额")
    private BigDecimal hiddenCommission = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠运费金额(原始单价上进行优惠)")
    private BigDecimal discountFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠运费总金额")
    private BigDecimal discountFreightTotalPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠清关费金额(原始单价上进行优惠)")
    private BigDecimal discountClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "活动优惠清关费总金额")
    private BigDecimal discountClearanceFeeTotalPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "液体清关费单价")
    private BigDecimal liquidClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "液体运费单价")
    private BigDecimal liquidFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "带电清关费单价")
    private BigDecimal electrifiedClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "带电运费单价")
    private BigDecimal electrifiedFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "品牌清关费单价")
    private BigDecimal brandClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "品牌运费单价")
    private BigDecimal brandFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "包装清关费单价")
    private BigDecimal packagingClearanceFeePrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "包装运费单价")
    private BigDecimal packagingFreightPrice = BigDecimal.ZERO;

    @ApiModelProperty(value = "是否特价佣金设置")
    private Boolean specialCommission = Boolean.FALSE;

    @ApiModelProperty(value = "价格有效期开始")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date startTime;

    @ApiModelProperty(value = "价格有效期结束")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date endTime;

    public void setBrand(Integer brand) {
        this.brand = brand;
        this.brandType = brand;
    }


    public void setOneSeaFreight(BigDecimal oneSeaFreight) {
        this.oneSeaFreight = oneSeaFreight;
        if (Objects.nonNull(oneSeaFreight) && Objects.nonNull(this.plainCommission) && Objects.nonNull(this.hiddenCommission)) {
            // 原价=销售价-明佣
            // 成本价=原价-暗佣
            this.costSeaFreight = this.oneSeaFreight.subtract(this.plainCommission).subtract(hiddenCommission);
        }
    }

    public void setPlainCommission(BigDecimal plainCommission) {
        this.plainCommission = plainCommission;
        if (Objects.nonNull(this.oneSeaFreight) && Objects.nonNull(plainCommission) && Objects.nonNull(this.hiddenCommission)) {
            // 原价=销售价-明佣
            // 成本价=原价-暗佣
            this.costSeaFreight = this.oneSeaFreight.subtract(this.plainCommission).subtract(hiddenCommission);
        }
    }

    public void setHiddenCommission(BigDecimal hiddenCommission) {
        this.hiddenCommission = hiddenCommission;
        if (Objects.nonNull(this.oneSeaFreight) && Objects.nonNull(this.plainCommission) && Objects.nonNull(hiddenCommission)) {
            // 原价=销售价-明佣
            // 成本价=原价-暗佣
            this.costSeaFreight = this.oneSeaFreight.subtract(this.plainCommission).subtract(hiddenCommission);
        }
    }
}
