package cn.iocoder.yudao.module.sale.vo.offer.prod;

import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import io.swagger.annotations.*;

import com.alibaba.excel.annotation.ExcelProperty;

import javax.validation.constraints.NotNull;

/**
 * 报价商品详情 Excel VO
 *
 * @author 系统管理员
 */
@Data
public class OfferProdExcelVO {

    @ExcelProperty("主键ID")
    private Long offerProdId;

    @ExcelProperty(value = "运输id")
    @NotNull(message = "运输id不能为空")
    private Integer transportId;

    @ExcelProperty("报价单ID")
    private Long offerId;

    @ExcelProperty("商品ID")
    private Long prodId;

    @ExcelProperty(value = "产品中文标题")
    private String prodTitleZh;

    @ExcelProperty(value = "产品英文标题")
    private String prodTitleEn;

    @ExcelProperty(value = "商品第三方链接")
    private String link;

    @ExcelProperty("线路id")
    private Long lineId;

    @ExcelProperty("渠道id")
    private Long channelId;

    @ExcelProperty("产品类型")
    private Integer productType;

    @ExcelProperty("是否品牌")
    private Integer brand;
    @ExcelProperty(value = "品牌(0 无牌 1 有牌 2 中性)")
    private Integer brandType;

    @ExcelProperty("件数")
    private Integer num;

    @ExcelProperty("单位")
    private String unit;

    @ExcelProperty("箱规")
    private String boxGauge;

    @ExcelProperty("体积")
    private BigDecimal volume;

    @ExcelProperty("重量")
    private BigDecimal weight;

    @ExcelProperty("货值")
    private BigDecimal worth;

    @ExcelProperty(value = "货值金额单位ID(默认美元)")
    private Integer worthCurrency;

    @ExcelProperty(value = "商品特性(可选多个特性ids)")
    private String prodAttrIds;

    @ExcelProperty(value = "是否特价")
    private Boolean specialPriceType;

    @ExcelProperty(value = "运费单价(成交)")
    private BigDecimal oneSeaFreight;

    @ExcelProperty(value = "销售价运费")
    private BigDecimal saleSeaFreight;

    @ExcelProperty(value = "成本价运费")
    private BigDecimal costSeaFreight;

    @ExcelProperty(value = "运费(小计)")
    private BigDecimal seaFreight;

    @ExcelProperty(value = "原来运费(原始)")
    private BigDecimal originalSeaFreight;

    @ExcelProperty("运费用金额单位ID")
    private Integer seaFreightCurrency;

    @ExcelProperty("运费体积单位")
    private Long seaFreightVolume;

    @ExcelProperty(value = "清关费单价(成交)")
    private BigDecimal oneClearanceFreight;

    @ExcelProperty(value = "清关费(小计)")
    private BigDecimal clearanceFreight;

    @ExcelProperty(value = "原来清关费(原始)")
    private BigDecimal originalClearanceFreight;

    @ExcelProperty("清关费用金额单位ID")
    private Integer clearanceFreightCurrency;

    @ExcelProperty("清关费体积单位")
    private Long clearanceFreightVolume;

    @ExcelProperty("预计销售额")
    private BigDecimal estimatedSales;

    @ExcelProperty("是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ExcelProperty(value = "计费方式：0 运费/清关费 1 全包价")
    private Integer charging;

    @ExcelProperty(value = "数量")
    private Integer quantity;

    @ExcelProperty(value = "状态：0 正常 1 审批中 2 审批通过 3 审批未通过")
    private Integer status;

    @ExcelProperty("价格有效期开始")
    private Date startTime;

    @ExcelProperty("价格有效期结束")
    private Date endTime;

    @ExcelProperty("创建时间")
    private Date createTime;



    @ExcelProperty("快递单号")
    private String expressNo;

    @ExcelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ExcelProperty("明佣金额")
    private BigDecimal plainCommission = BigDecimal.ZERO;

    @ApiModelProperty("暗佣金额")
    private BigDecimal hiddenCommission = BigDecimal.ZERO;

    public void setBrand(Integer brand) {
        this.brand = brand;
        this.brandType = brand;
    }

    public void setOneSeaFreight(BigDecimal oneSeaFreight) {
        this.oneSeaFreight = oneSeaFreight;
        if (Objects.nonNull(this.hiddenCommission)) {
            this.costSeaFreight = oneSeaFreight.subtract(this.hiddenCommission);
            if (Objects.nonNull(this.plainCommission)){
                this.saleSeaFreight = oneSeaFreight.subtract(hiddenCommission).subtract(this.plainCommission);
            }
        }
    }

    public void setPlainCommission(BigDecimal plainCommission) {
        this.plainCommission = plainCommission;
        this.saleSeaFreight = this.oneSeaFreight.subtract(this.hiddenCommission).subtract(plainCommission);
    }

    public void setHiddenCommission(BigDecimal hiddenCommission) {
        this.hiddenCommission = hiddenCommission;
        if (Objects.nonNull(this.oneSeaFreight)) {
            this.costSeaFreight = this.oneSeaFreight.subtract(hiddenCommission);
            if (Objects.nonNull(this.plainCommission)){
                this.saleSeaFreight = this.oneSeaFreight.subtract(hiddenCommission).subtract(this.plainCommission);
            }
        }

    }
}
