package cn.iocoder.yudao.module.sale.vo.offer.prod;

import com.alibaba.excel.annotation.ExcelProperty;
import lombok.*;

import java.math.BigDecimal;
import java.util.*;
import io.swagger.annotations.*;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@ApiModel(value = "管理后台 - 报价商品详情 Excel 导出 Request VO", description = "参数和 OfferProdPageReqVO 是一致的")
@Data
public class OfferProdExportReqVO {


    @ApiModelProperty(value = "运输id", required = true)
    @NotNull(message = "运输id不能为空")
    private Integer transportId;

    @ApiModelProperty(value = "报价单ID")
    private Long offerId;

    @ApiModelProperty(value = "商品ID")
    private Long prodId;

    @ApiModelProperty(value = "产品中文标题")
    private String prodTitleZh;

    @ApiModelProperty(value = "产品英文标题")
    private String prodTitleEn;

    @ApiModelProperty(value = "商品第三方链接")
    private String link;

    @ApiModelProperty(value = "线路id")
    private Long lineId;

    @ApiModelProperty(value = "渠道id")
    private Long channelId;

    @ApiModelProperty(value = "产品类型")
    private Integer productType;

    @ApiModelProperty(value = "是否品牌")
    private Integer brand;

    @ApiModelProperty(value = "件数")
    private Integer num;

    @ApiModelProperty(value = "单位")
    private String unit;

    @ApiModelProperty(value = "箱规")
    private String boxGauge;

    @ApiModelProperty(value = "体积")
    private BigDecimal volume;

    @ApiModelProperty(value = "重量")
    private BigDecimal weight;

    @ApiModelProperty(value = "货值")
    private BigDecimal worth;

    @ApiModelProperty(value = "货值金额单位ID(默认美元)")
    private Integer worthCurrency;

    @ApiModelProperty(value = "商品特性(可选多个特性ids)")
    private String prodAttrIds;

    @ApiModelProperty(value = "是否特价")
    private Boolean specialPriceType;

    @ApiModelProperty(value = "运费单价(成交)")
    private BigDecimal oneSeaFreight;

    @ApiModelProperty(value = "销售价运费")
    private BigDecimal saleSeaFreight;

    @ApiModelProperty(value = "成本价运费")
    private BigDecimal costSeaFreight;

    @ApiModelProperty(value = "运费(小计)")
    private BigDecimal seaFreight;

    @ApiModelProperty(value = "原来运费(原始)")
    private BigDecimal originalSeaFreight;

    @ApiModelProperty(value = "运费用金额单位ID")
    private Integer seaFreightCurrency;

    @ApiModelProperty(value = "运费体积单位")
    private Long seaFreightVolume;

    @ApiModelProperty(value = "清关费单价(成交)")
    private BigDecimal oneClearanceFreight;

    @ApiModelProperty(value = "清关费(小计)")
    private BigDecimal clearanceFreight;

    @ApiModelProperty(value = "原来清关费(原始)")
    private BigDecimal originalClearanceFreight;

    @ApiModelProperty(value = "清关费用金额单位ID")
    private Integer clearanceFreightCurrency;

    @ApiModelProperty(value = "清关费体积单位")
    private Long clearanceFreightVolume;

    @ApiModelProperty(value = "预计销售额")
    private BigDecimal estimatedSales;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ApiModelProperty(value = "计费方式：0 运费/清关费 1 全包价")
    private Integer charging;

    @ApiModelProperty(value = "数量")
    private Integer quantity;

    @ApiModelProperty(value = "状态：0 正常 1 审批中 2 审批通过 3 审批未通过")
    private Integer status;


    @ApiModelProperty("快递单号")
    private String expressNo;

    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明+暗")
    private Integer commissionType;

    @ApiModelProperty("明佣金额")
    private BigDecimal plainCommission = BigDecimal.ZERO;

    @ApiModelProperty("暗佣金额")
    private BigDecimal hiddenCommission = BigDecimal.ZERO;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始价格有效期开始")
    private Date beginStartTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束价格有效期开始")
    private Date endStartTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始价格有效期结束")
    private Date beginEndTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束价格有效期结束")
    private Date endEndTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始创建时间")
    private Date beginCreateTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束创建时间")
    private Date endCreateTime;


    public void setOneSeaFreight(BigDecimal oneSeaFreight) {
        this.oneSeaFreight = oneSeaFreight;
        if (Objects.nonNull(this.hiddenCommission)) {
            this.costSeaFreight = oneSeaFreight.subtract(this.hiddenCommission);
            if (Objects.nonNull(this.plainCommission)){
                this.saleSeaFreight = oneSeaFreight.subtract(hiddenCommission).subtract(this.plainCommission);
            }
        }
    }

    public void setPlainCommission(BigDecimal plainCommission) {
        this.plainCommission = plainCommission;
        this.saleSeaFreight = this.oneSeaFreight.subtract(this.hiddenCommission).subtract(plainCommission);
    }

    public void setHiddenCommission(BigDecimal hiddenCommission) {
        this.hiddenCommission = hiddenCommission;
        if (Objects.nonNull(this.oneSeaFreight)) {
            this.costSeaFreight = this.oneSeaFreight.subtract(hiddenCommission);
            if (Objects.nonNull(this.plainCommission)){
                this.saleSeaFreight = this.oneSeaFreight.subtract(hiddenCommission).subtract(this.plainCommission);
            }
        }

    }

}

