package cn.iocoder.yudao.module.sale.vo.offerApproval;

import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
* 报价单审核详情 Response VO
* @author 系统管理员
*/
@Data
@ApiModel("管理后台 - 报价单审核详情 Response VO")
public class OfferApprovalBackInfoVO {

    @ExcelProperty("主键ID")
    @ApiModelProperty(value = "主键ID", required = true)
    private Long offerApprovalId;

    @ExcelProperty("报价单ID")
    @ApiModelProperty(value = "报价单ID", required = true)
    private Long offerId;

    @ExcelProperty("报价商品ID")
    @ApiModelProperty(value = "报价商品ID", required = true)
    private Long offerProdId;

    @ExcelProperty("流程表单实例ID")
    @ApiModelProperty(value = "流程表单实例ID")
    private String formId;

    @ExcelProperty("类型：1 特价优惠 2 佣金申请")
    @ApiModelProperty(value = "类型：1 特价优惠 2 佣金申请")
    private Integer type;

    @ExcelProperty("跟进业务员")
    @ApiModelProperty(value = "跟进业务员")
    private String followUpSalesmanId;

    @ExcelProperty("审核内容详情")
    @ApiModelProperty(value = "审核内容详情")
    private String details;

    @ExcelProperty("状态：1 处理中 2 通过 3 不通过 4 取消")
    @ApiModelProperty(value = "状态：1 处理中 2 通过 3 不通过 4 取消")
    private Integer status;

    @ExcelProperty("处理结果")
    @ApiModelProperty(value = "处理结果")
    private String processingResults;

    @ExcelProperty("创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "创建时间", required = true)
    private Date createTime;

    @ExcelProperty("结束人")
    @ApiModelProperty(value = "结束人")
    private String finishBy;

    @ExcelProperty("结束时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束时间")
    private Date finishTime;


    @ApiModelProperty(value = "商品ID", required = true)
    @NotNull(message = "商品ID不能为空")
    private Long prodId;

    @ApiModelProperty(value = "产品中文标题")
    private String prodTitleZh;

    @ApiModelProperty(value = "产品英文标题")
    private String prodTitleEn;

    @ApiModelProperty(value = "始发地ID")
    private Long departureId;

    @ApiModelProperty(value = "始发地")
    private String departureName;

    @ApiModelProperty(value = "目的地ID")
    private Long objectiveId;

    @ApiModelProperty(value = "目的地")
    private String objectiveName;

    @ApiModelProperty(value = "运输id", required = true)
    @NotNull(message = "运输id不能为空")
    private Integer transportId;

    @ApiModelProperty(value = "线路id", required = true)
    @NotNull(message = "线路id不能为空")
    private Long lineId;

    @ApiModelProperty(value = "渠道id")
    private Long channelId;

    @ApiModelProperty(value = "产品类型")
    private Integer productType;

    @ApiModelProperty(value = "是否预付：0 到付 1 预付")
    private Integer isPayAdvance;

    @ApiModelProperty(value = "计费方式：0 运费/清关费 1 全包价")
    private Integer charging;

    @ApiModelProperty(value = "数量")
    private Integer quantity;

    @ApiModelProperty(value = "原运费(佣金-> 原始线路单价 优惠->原销售单价)")
    private BigDecimal orgSeaFreight;

    @ApiModelProperty(value = "成本运费")
    private BigDecimal costSeaFreight;

    @ApiModelProperty(value = "原清关费(佣金-> 原始线路单价 优惠->原销售单价)")
    private BigDecimal orgClearanceFreight;

    @ApiModelProperty(value = "成本清关费")
    private BigDecimal costClearanceFreight;

    @ApiModelProperty(value = "小计运费")
    private BigDecimal subtotalFreight = BigDecimal.ZERO;

    @ApiModelProperty(value = "销售运费(佣金->销售运费单价 优惠->新的销售单价)")
//    @NotNull(message = "运费不能为空")
    private BigDecimal seaFreight;

    @ApiModelProperty(value = "运费用金额单位ID")
//    @NotNull(message = "运费用金额单位ID不能为空")
    private Long seaFreightCurrency;

    @ApiModelProperty(value = "运费体积单位ID")
//    @NotNull(message = "运费体积单位ID不能为空")
    private Long seaFreightVolume;

    @ApiModelProperty(value = "小计清关费")
    private BigDecimal subtotalClearanceFreight = BigDecimal.ZERO;

    @ApiModelProperty(value = "销售清关费(佣金->销售运费单价 优惠->新的销售单价)")
//    @NotNull(message = "清关费不能为空")
    private BigDecimal clearanceFreight;

    @ApiModelProperty(value = "清关费用金额单位ID")
//    @NotNull(message = "清关费用金额单位ID不能为空")
    private Long clearanceFreightCurrency;

    @ApiModelProperty(value = "清关费体积单位ID")
//    @NotNull(message = "清关费体积单位ID不能为空")
    private Long clearanceFreightVolume;

//    @ApiModelProperty(value = "佣金ID")
//    private Long commissionId;

    @ApiModelProperty(value = "原佣金类型：0 无 1 明佣 2 暗佣 3 明佣+暗佣")
    private Integer orgCommissionType = 0;

    @ApiModelProperty(value = "原明佣金额(货币、单位必须一致,线路产品单价既是成本价，佣金加单价就时销售价格)")
    private BigDecimal orgLightCommissionAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "原暗佣金额")
    private BigDecimal orgShadeCommissionAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "佣金类型：0 无 1 明佣 2 暗佣 3 明佣+暗佣")
    private Integer commissionType = 0;

    @ApiModelProperty(value = "明佣金额(货币、单位必须一致,线路产品单价既是成本价，佣金加单价就时销售价格)")
    private BigDecimal lightCommissionAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "暗佣金额")
    private BigDecimal shadeCommissionAmount = BigDecimal.ZERO;

    @ApiModelProperty(value = "审批状态: 1 审批中  2 审批通过 3 审批拒绝 4 取消")
    private Integer applyStatus = 0;

    @ApiModelProperty(value = "审批结果")
    private String applyResult;

}
